$(document).ready(async function () {

  initTooltips();
  initServiceId();

  //change image select after close modal
  $('body').on('hide.bs.modal', '#fileManagerModal', function () {
    createImageList('#customImage', true, serviceArrTmp[activeItem].image);
  });

  //colorScheme buttons
  $('.chooseColorScheme').on('click', function () {
    $('.chooseColorScheme').css('border-color', 'transparent');
    var colorScheme = $(this).attr('data-value');
    $('.chooseColorScheme[data-value="' + colorScheme + '"]').css('border-color', '#508cff');
    $('#colorScheme').val(colorScheme);
    return false;
  });

  //setPtz buttons
  $('#setNorthPtz').on('click', function () { setPtz('north'); });
  $('#setEastPtz').on('click', function () { setPtz('east'); });
  $('#setWestPtz').on('click', function () { setPtz('west'); });
  $('#setSouthPtz').on('click', function () { setPtz('south'); });

  //compass type button
  $('#openModal').on('click', function () {
    $('[data-toggle="tooltip"]').tooltip('hide');
    var compassType = $('#compassType').val();
    if (compassType === 'map') $('#getMapImage').modal();
    if (compassType === 'image') showImageManagerCallback();
  });

  //img with custom picture of map
  $('#imgPosition').on('click', function (e) {
    var cursorX = e.pageX;
    var cursorY = e.pageY;
    var leftMargin = $(this).offset().left;
    var topMargin = $(this).offset().top;
    renderPoint($(this), cursorX, cursorY, leftMargin, topMargin);
  });

  // Fill form
  initLayerZindex(serviceArrTmp[activeItem].zIndex || 0);

  $('#compassCustomName')
    .val(serviceArrTmp[activeItem].customName || '')
    .keyup(onCustomNameKeyUp)
    .attachValidation({
      pattern: '^[A-Za-z0-9 -]{0,}$',
      invalidMsg: 'Only chars A-Z, a-z, 0-9, white space and dash are allowed.',
      disableBtnId: 'saveBtn'
    });
  renderCompassTypeSideBtn(serviceArrTmp[activeItem].type);

  await createImageList('#customImage', true, serviceArrTmp[activeItem].image);

  if (serviceArrTmp[activeItem].image !== '' && serviceArrTmp[activeItem].cameraPosX && serviceArrTmp[activeItem].cameraPosY) {
    renderCompassTypeComponents(serviceArrTmp[activeItem].type, serviceArrTmp[activeItem].image);
    renderPointStart($('#imgPosition'), serviceArrTmp[activeItem].cameraPosX, serviceArrTmp[activeItem].cameraPosY);
  }
  $('#compassPositionOnImageX').val(serviceArrTmp[activeItem].cameraPosX);
  $('#compassPositionOnImageY').val(serviceArrTmp[activeItem].cameraPosY);

  renderColorScheme(serviceArrTmp[activeItem].type);
  if (serviceArrTmp[activeItem].colorScheme !== '') {
    $('#colorScheme').val(serviceArrTmp[activeItem].colorScheme);
    var colorScheme = $('#colorScheme').val();
    $('.chooseColorScheme[data-value="' + colorScheme + '"]').css('border-color', '#508cff');
  }

  $('#compassPtz').val(serviceArrTmp[activeItem].northPan);
  $('#compassOffsetX').val(serviceArrTmp[activeItem].pos_x);
  $('#compassOffsetY').val(serviceArrTmp[activeItem].pos_y);
  $('#compassScale').val(serviceArrTmp[activeItem].scale * 100);

  const resolutionId = '#compassResolution';
  const videoChannelId = '#compassVideoChannel';
  const videoPreviewSelectId = '#compassPreviewVideoChannel';
  createResolutionList(resolutionId);
  onResolutionChange(resolutionId, videoChannelId);
  createPositionList('#compassPosition', serviceArrTmp[activeItem].coordSystem);
  await createTriggeringElement('#compassAutomationTypeContainer', serviceArrTmp[activeItem].automationType, serviceArrTmp[activeItem].schedule, false);
  onVideoChannelChange(resolutionId, videoChannelId, videoPreviewSelectId);
  createVideoChannelElement(videoChannelId, serviceArrTmp[activeItem].cameraList).then(function () {
    // Callbacks for changes
    $('.form-control').change(formChangedCallback);
    $('.chooseColorScheme').on('click', formChangedCallback);
    onPreviewVideoChannelChange(videoPreviewSelectId, resolutionId);

    formChangedCallback();

    let currentCameraResolutionObject = cameraResolutions[parseInt($('#compassResolution').val())];
    showImageFromCamera('imagePreviewCamera', currentCameraResolutionObject, $(videoPreviewSelectId).val());
  });

  //compass type select
  $('#compassType').on('change', function () {
    let compassType = $(this).val();
    renderColorScheme(compassType);
    renderCompassTypeSideBtn(compassType);
    renderCompassTypeComponents(compassType, serviceArrTmp[activeItem].image);
  });

  //custom image select
  $('#customImage').on('change', function () {
    let customImage = $('#customImage option:selected').val();
    renderCompassTypeComponents($('#compassType').val(), customImage);
  });

  createScheduleBtn();
});

function renderCompassTypeSideBtn(type) {
  $('#compassType').val(type);
  var button = $('#openModal');
  if (type === 'image') {
    button.text('Image manager');
    button.attr('disabled', false);
  }
  if (type === 'map') {
    button.text('Edit map');
    button.attr('disabled', false);
  }
  if (type === 'compass') {
    button.text('');
    button.attr('disabled', true);
  }
}

function renderColorScheme(type) {
  if (type === 'image' || type === 'map') $('#colorSchemaDiv').hide();
  if (type === 'compass') $('#colorSchemaDiv').show();
}

function renderCompassTypeComponents(type, image) {
  serviceArrTmp[activeItem].type = type;
  $('#selectCustomImgFormRow').hide();
  $('#compassPositionFormRow').hide();

  if (type === 'image') {
    $('#selectCustomImgFormRow').show();
  }
  if (type === 'image' || type === 'map') {
    $('#compassPositionFormRow').show();
  }

  if (image !== '' && image !== undefined) {
    let containMapImage = $('#customImage option[value*="map_picture_' + serviceArrTmp[activeItem].id + '.png"]').length === 1;
    let imgSrc = '';
    if (type === 'map' && !containMapImage) {
      $('#imgPosition').attr('src', imgSrc);
    }
    if (type === 'map' && containMapImage) {
      image = $('#customImage option[value*="map_picture_' + serviceArrTmp[activeItem].id + '.png"]').attr('value');
      imgSrc = '/local/camoverlay/api/image.cgi?path=' + encodeURIComponent(image) + '&time=' + new Date().getTime();
      $('#imgPosition').attr('src', imgSrc);
    }
    if (type === 'image') {
      imgSrc = '/local/camoverlay/api/image.cgi?path=' + encodeURIComponent(image) + '&time=' + new Date().getTime();
      $('#imgPosition').attr('src', imgSrc);
    }

    serviceArrTmp[activeItem].image = image;
    $('#customImage option[value="' + image + '"]').attr('selected');
  }
}

function renderPointStart(element, posX, posY) {
  var iconSize = 20;
  element.parent().find('.point').remove();
  element.parent().append('<div class="point" style="width: ' + iconSize + 'px; height: ' + iconSize + 'px; border: 2px solid #FFF; box-shadow: 0 0 10px #000; border-radius: 50%; background-color: #0950ff; position: absolute; top:' + posY + 'px; left: ' + posX + 'px;"></div>');
}

function renderPoint(element, cursorX, cursorY, leftMargin, topMargin) {
  var iconSize = 20;
  var posY = (cursorY - (iconSize / 2)) - topMargin;
  var posX = (cursorX + (iconSize / 2)) - leftMargin;

  element.parent().find('.point').remove();
  element.parent().append('<div class="point" style="width: ' + iconSize + 'px; height: ' + iconSize + 'px; border: 2px solid #FFF; box-shadow: 0 0 10px #000; border-radius: 50%; background-color: #0950ff; position: absolute; top:' + posY + 'px; left: ' + posX + 'px;"></div>');

  $('#compassPositionOnImageY').val(Math.ceil(posY));
  $('#compassPositionOnImageX').val(Math.ceil(posX));
  formChangedCallback();
}

async function setPtz(world_side) {
  let position = await ptzManager.getPosition();
  let pan = -1 * parseFloat(position['pan']);
  if (world_side === 'east') pan = pan + 90;
  if (world_side === 'west') pan = pan - 90;
  if (world_side === 'south') pan = pan - 180;
  $('#compassPtz').val(pan);
  formChangedCallback();
}

async function uploadImage(externalImg) {
  let imgName = 'map_picture_' + serviceArrTmp[activeItem].id + '.png';

  const response = await requestManager.get(externalImg);
  const imgBinaryDataResult = await response.blob();

  // Create form
  let formTmp = new FormData();
  formTmp.append("target", 'flash');
  formTmp.append("uploadedFile", imgBinaryDataResult, imgName);

  try {
    await camOverlayApi.uploadImage(formTmp);
  } catch (error) {
    throw new Error(error.message);
  }

  return `file:///usr/local/packages/camoverlay/localdata/user_images/${imgName}`;
}

async function formChangedCallback() {

  serviceArrTmp[activeItem].customName = $('#compassCustomName').val();
  let currentCameraResolutionObject = cameraResolutions[parseInt($('#compassResolution').val())];
  serviceArrTmp[activeItem].width = currentCameraResolutionObject.width;
  serviceArrTmp[activeItem].height = currentCameraResolutionObject.height;
  await saveLayerZindex($('#layerZindex').val());
  serviceArrTmp[activeItem].northPan = parseFloat($('#compassPtz').val());
  serviceArrTmp[activeItem].type = $('#compassType').val();
  serviceArrTmp[activeItem].pos_x = parseInt($('#compassOffsetX').val());
  serviceArrTmp[activeItem].pos_y = parseInt($('#compassOffsetY').val());
  serviceArrTmp[activeItem].scale = parseFloat($('#compassScale').val()) / 100;
  serviceArrTmp[activeItem].colorScheme = $('#colorScheme').val();
  serviceArrTmp[activeItem].coordSystem = $('#compassPosition').val();
  serviceArrTmp[activeItem].automationType = $('#compassAutomationTypeContainer input[type="radio"]:checked').val();
  serviceArrTmp[activeItem].cameraPosX = parseInt($('#compassPositionOnImageX').val());
  serviceArrTmp[activeItem].cameraPosY = parseInt($('#compassPositionOnImageY').val());

  serviceArrTmp[activeItem].cameraList = getSelectedVideoChannelValues($('#compassVideoChannel'));

  showSaveWarning();
}