class Switch {

  serviceId = null;
  element = null;
  toggle = null;
  configurationTmp = null;

  /**
   * Render html container with input
   * @returns {Promise<void>}
   */
  render() {
    const label_html = document.createElement('label');
    label_html.className = 'switch';
    const span_html = document.createElement('span');
    span_html.className = 'slider round';
    label_html.appendChild(span_html);
    this.element.appendChild(label_html);
  }

  /**
   * @param {boolean} value
   */
  setValue(value) {
    this.toggle.setAttribute('checked', Boolean(value));
    serviceArrTmp[this.serviceId].enabled = value ? 1 : 0;
    serviceArr[this.serviceId].enabled = value ? 1 : 0;
    saveBtnCallback(true, serviceArr);
  }

  /**
   * @param {string} element_id
   */
  destroy(element_id) {
    this.element = $(element_id)[0];
    this.toggle = this.element.getElementsByTagName('label')[0];
    if (this.toggle !== undefined) {
      this.toggle.removeEventListener('click', () => {});
    }
    $(this.element).html('');
  }

  /**
   * @param {string} element_id
   * @returns {Promise<void>}
   */
  async init(element_id, defaultValue, config_tmp) {
    this.configurationTmp = config_tmp;
    this.element = $(element_id)[0];
    this.render();

    this.toggle = this.element.getElementsByTagName('label')[0];
    this.serviceId = Number($(this.toggle).closest('.serviceListItem').attr('data-order'));

    this.toggle.removeEventListener('click', () => {});
    this.toggle.addEventListener('click', async () => {
      this.serviceId = Number($(this.toggle).closest('.serviceListItem').attr('data-order'));
      const origin_value = parseBool(this.toggle.getAttribute('checked'));
      let value = !origin_value;
      if (!origin_value) {
        const allow_turn_on = await this.configurationTmp.canEnableAnotherService();
        if (!allow_turn_on) {
          const alert_message = await this.configurationTmp.getMaxAllowedMessage();
          showMessage('info', alert_message, false, 'maxAllowedServices');
        }
      }
      this.setValue(value);
    });

    const init_allow_turn_on = await this.configurationTmp.exceedsMaxServiceCount();
    if (init_allow_turn_on) {
      const init_alert_message = await this.configurationTmp.getMaxAllowedMessage();
      showMessage('info', init_alert_message, false, 'maxAllowedServices');
    }
    this.setValue(Boolean(defaultValue));
  }
}