var activeOverlay = 0;
var added = false;

$(document).ready(async function () {

  initTooltips();
  initServiceId();

  $('#customScale').on('keyup change', function () {
    validateRange($(this), {
      minVal: 30,
      maxVal: 200,
      invalidMsg: 'Must be from interval 30-200%.',
      disableBtnId: 'saveOverlayBtn'
    });
  });

  $('#customDuration').on('keyup change', function () {
    validateRange($(this), {
      minVal: 0,
      maxVal: 99999,
      invalidMsg: 'Must be from interval 0-99999s.',
      disableBtnId: 'saveOverlayBtn'
    });
  });

  $('#customFPS').on('keyup change', function () {
    validateRange($(this), {
      minVal: 0,
      maxVal: 30,
      invalidMsg: 'The value must be 1 to 30.',
      disableBtnId: 'saveOverlayBtn'
    });
  });

  // Fill form
  initLayerZindex(serviceArrTmp[activeItem].zIndex || 0);

  $('#customCustomName')
    .val(serviceArrTmp[activeItem].customName || '')
    .keyup(onCustomNameKeyUp)
    .attachValidation({
      pattern: '^[A-Za-z0-9 -]{0,}$',
      invalidMsg: 'Only chars A-Z, a-z, 0-9, white space and dash are allowed.',
      disableBtnId: 'saveBtn'
    });
  const resolutionId = '#customResolution';
  const videoChannelId = '#customVideoChannel';
  const videoPreviewSelectId = '#customPreviewVideoChannel';
  createResolutionList(resolutionId);
  onResolutionChange(resolutionId, videoChannelId);

  await createTriggeringElement('#customAutomationTypeContainer', serviceArrTmp[activeItem].automationType, serviceArrTmp[activeItem].schedule, true);
  $('#customAutomationTypeContainer').find('input[type="radio"]').change(handleAutomationType);

  onVideoChannelChange(resolutionId, videoChannelId, videoPreviewSelectId);
  createVideoChannelElement(videoChannelId, serviceArrTmp[activeItem].cameraList).then(function () {
    onPreviewVideoChannelChange(videoPreviewSelectId, resolutionId);
    // Callbacks for changes
    $('#mainForm .form-control').change(formChangedCallback);
    $('#saveOverlayBtn').click(saveEditedOverlayCallback);
    $('#editOverlayModalDismiss').click(cancelEditedOverlayCallback);
    $('#cancelOverlayBtn').click(cancelEditedOverlayCallback);
    $('#addOverlayBtn').click(addOverlayClickedCallback);
    $('#customImageManagerBtn').click(showImageManagerCallback);
    $('#customImgPath').change(customImagePathChanged);

    formChangedCallback();

    let currentCameraResolutionObject = cameraResolutions[parseInt($('#customResolution').val())];
    showImageFromCamera('imagePreviewCamera', currentCameraResolutionObject, $(videoPreviewSelectId).val());
  });

  showOverlayList();

  createScheduleBtn();
});

async function fileExists(overlayList) {
  let fileListExists = [];
  if (overlayList != undefined && overlayList.length > 0) {
    for (let i = 0; i < overlayList.length; i++) {
      let imgSrc = getImageSrc(overlayList[i].imgPath);
      let existResult = await imageExists(imgSrc);
      fileListExists.push(existResult);
    }
  }
  return fileListExists;
}

function handleAutomationType(event) {
  const automationTypeValue = $(event.target).val();
  $('.automationTypeId').removeClass('d-block').addClass('d-none');
  const automationTypeId = automationTypeValue !== 'manual' ? 'time' : 'manual';
  $('.automationTypeId[data-type-id="' + automationTypeId + '"]').removeClass('d-none').addClass('d-block');

  handleAutomationTypeLabel(automationTypeValue);
}

async function formChangedCallback() {
  serviceArrTmp[activeItem].customName = $('#customCustomName').val();

  if (serviceArrTmp[activeItem].name == "custom") {
    serviceArrTmp[activeItem].name = "images";
  }
  let currentCameraResolutionObject = cameraResolutions[parseInt($('#customResolution').val())];
  serviceArrTmp[activeItem].width = currentCameraResolutionObject.width;
  serviceArrTmp[activeItem].height = currentCameraResolutionObject.height;
  await saveLayerZindex($('#layerZindex').val());
  serviceArrTmp[activeItem].automationType = $('#customAutomationTypeContainer input[type="radio"]:checked').val();

  serviceArrTmp[activeItem].cameraList = getSelectedVideoChannelValues($('#customVideoChannel'));

  showSaveWarning();
}

/** Create input option HTML
 * @param {number} id
 * @param {boolean} isActive
 * @returns {string}
 */
function getOverlayRadioInputHTML(id, isActive) {
  const checked = isActive ? 'checked' : '';
  const labelText = isActive ? 'active' : 'choose';
  const classActive = isActive ? 'overlay_active' : '';
  const radioInput = '<input type="radio" id="overlayEnabled' + id + '" data-id="' + id + '" class="radio_overlay_active form-check-input" ' + checked + ' />';
  const radioInputLabel = '<label class="radio_overlay_active_label form-check-label ' + classActive + '" for="overlayEnabled' + id + '">' + labelText + '</label>';
  return '<div class="form-group custom-control form-check mb-0">' + radioInput + radioInputLabel + '</div>';
}

function RadioInput(elem) {

  const disable = () => handleDisabled(true);
  const enable = () => handleDisabled(false);
  /**
   * @param {boolean} disable
   */
  const handleDisabled = (disable) => {
    elem.disabled = disable;
  }

  const check = () => handleCheck(true);
  const uncheck = () => handleCheck(false);
  /**
   * @param {boolean} check
   */
  const handleCheck = (check) => {
    elem.checked = check;
    if (check) {
      elem.setAttribute('checked', 'checked');
    } else {
      elem.removeAttribute('checked');
    }
    setLabel(check);
    handleSaveOverlay(elem, check);
  }

  /**
   * @param {boolean} check
   */
  const setLabel = (check) => {
    const label = RadioLabel($(elem).next('label'));
    label.setText(check ? 'active' : 'choose');
    label.setActive(check);
  }

  return {
    disable,
    enable,
    check,
    uncheck,
  }
}

/**
 * @param elem
 * @param {boolean} isActive
 */
function handleSaveOverlay(elem, isActive) {
  const elemId = $(elem).data('id');
  const tmpOverlayList = serviceArrTmp[activeItem].overlayList[elemId];
  const overlayList = serviceArr[activeItem].overlayList[elemId];
  tmpOverlayList.active = isActive;
  const existsInOverlayList = tmpOverlayList === overlayList;
  if (!existsInOverlayList) {
    serviceArr[activeItem].overlayList.push(tmpOverlayList);
  }
  overlayList.active = isActive;
}

// need var, otherwise problems with redeclaring
var URL_LINK_ICON_TITLE = 'URL Link';
function getStorageIcon(imageSrc) {
  if (imageSrc.indexOf('SD_DISK') > -1) {
    return '<i class="ml-2 fas fa-sd-card text-muted" title="SD Card"></i>';
  } else if (imageSrc.indexOf('localdata') > -1) {
    return '<i class="ml-2 fas fa-hdd text-muted" title="Local Storage"></i>';
  } else if (imageSrc.indexOf('NetworkShare') > -1) {
    return '<i class="ml-2 fas fa-network-wired text-muted" title="Network Share"></i>';
  } else {
    return '<i class="ml-2 fas fa-link text-muted" title="' + URL_LINK_ICON_TITLE + '"></i>';
  }
}

async function showOverlayList() {
  const overlayListElement = $('#overlayList');
  overlayListElement.showSpinner();

  let overlayListHtml = '';
  if (serviceArrTmp[activeItem].overlayList != undefined && serviceArrTmp[activeItem].overlayList.length > 0) {

    let fileExistList = await fileExists(serviceArrTmp[activeItem].overlayList);

    overlayListHtml += '<tbody>';
    let overlayList = serviceArrTmp[activeItem].overlayList;

    for (let i = 0; i < overlayList.length; i++) {
      let imgSrc = getImageSrc(overlayList[i].imgPath);
      overlayListHtml += '<tr>';
      const notExistLabel = !fileExistList[i] ? '<span style="float: left;" class="badge badge-danger">File not found</span>' : '';
      const imagePreview = '<img id="imageManagerPreview" src="' + imgSrc + '" style="max-width: 60px;" />' + notExistLabel;
      overlayListHtml += '<td style="width: 1%;">' + imagePreview + '</td>';

      let storageIcon = getStorageIcon(imgSrc);
      const showFps = storageIcon.includes(`title="${URL_LINK_ICON_TITLE}"`);
      overlayListHtml += '<td>' + storageIcon + '</td>';

      overlayListHtml += '<td style="max-width: 150px; overflow: hidden; text-overflow: ellipsis; white-space: nowrap;">';
      overlayListHtml += '<span style="font-style: italic;">' + overlayList[i].imgName + '</span>';
      overlayListHtml += '</td>';

      const isAutomaticAutomation = serviceArrTmp[activeItem].automationType !== 'manual';
      const manualClass = !isAutomaticAutomation ? 'd-block' : 'd-none';
      const timeClass = isAutomaticAutomation ? 'd-block' : 'd-none';
      const inputRadioHtml = '<div class="automationTypeId ' + manualClass + '" data-type-id="manual">' + getOverlayRadioInputHTML(i, overlayList[i].active) + '</div>';
      const durationText = overlayList[i].duration ? overlayList[i].duration + 's' : '0s';
      const durationHtml = '<div class="automationTypeId hidden-xs ' + timeClass + '" data-type-id="time" style="text-align: center;">' + durationText + '</div>';
      overlayListHtml += '<td>' + inputRadioHtml + durationHtml + '</td>';

      const coordText = coordSystemToNiceName(overlayList[i].coordSystem) + '<br />';
      const posText = 'x: ' + overlayList[i].pos_x + ', y: ' + overlayList[i].pos_y;
      const scaleText = !isNaN(overlayList[i].scale) ? ', <br />scale: ' + convertFloatToPercentage(overlayList[i].scale) + '%' : '';
      const fpsText = showFps ? (overlayList[i].fps ? `, <br />${overlayList[i].fps}fps` : `, <br />1fps`) : '';
      overlayListHtml += '<td class="hidden-xs" style="text-align: center;">' + coordText + posText + scaleText + fpsText + '</td>';

      const editClassname = showFps ? 'show-fps' : '';
      overlayListHtml += '<td style="text-align: center;">';
      overlayListHtml += '<a href="#" id="moveOverlay' + i + '" class="overlayMoveUp icon formItem" style="margin-left: 0;"><span class="fas fa-arrow-circle-up" data-toggle="tooltip" title="Move Up"></span></a>';
      overlayListHtml += '<a href="#" id="moveOverlay' + i + '" class="overlayMoveDown icon formItem"><span class="fas fa-arrow-circle-down" data-toggle="tooltip" title="Move Down"></span></a>';
      overlayListHtml += '<a href="#" id="editOverlay' + i + '" class="overlayEdit formItem icon ' + editClassname + '"><span class="fas fa-edit" data-toggle="tooltip" title="Edit"></span></a>';
      overlayListHtml += '<a href="#" data-id="' + i + '" class="overlayCopy formItem icon"><span class="fas fa-copy" data-toggle="tooltip" title="Duplicate"></span></a>';
      overlayListHtml += '<a href="#" id="removeOverlay' + i + '" class="overlayRemove formItem icon"><span class="fas fa-trash" data-toggle="tooltip" title="Delete"></span></a>';
      overlayListHtml += '</td></tr>';
    }

    overlayListHtml += '</tbody>';

  } else {
    serviceArrTmp[activeItem].overlayList = [];
    overlayListHtml += '<p class="text-center">No overlay set.</p>';

  }

  overlayListElement.html(overlayListHtml);

  $('.overlayMoveUp').click(moveOverlayClicked);
  $('.overlayMoveDown').click(moveOverlayClicked);
  $('.overlayEdit').click(editOverlayClicked);
  $('.overlayRemove').click(removeOverlayClicked);
  $('.overlayCopy').click(copyOverlayClicked);
  $('.radio_overlay_active').click(handleOverlayActiveClick);
  handleAutomationTypeLabel(serviceArrTmp[activeItem].automationType);

  showSaveWarning();
}

function handleAutomationTypeLabel(automationType) {
  const tooltipText = automationType !== 'manual' ? 'To switch manually, change the triggering to Manual.' : 'For loop graphics playback, change the triggering to Automatic.';
  $('#automation_type_label').text(tooltipText);
}

function handleOverlayActiveClick(event) {
  $('.radio_overlay_active').each((index, item) => {
    RadioInput(item).uncheck();
  });
  RadioInput(event.target).check();

  saveBtnCallback(true);
}

function RadioLabel(item) {

  let setActive = (isActive) => {
    isActive ? item.addClass('overlay_active') : item.removeClass('overlay_active');
  }

  let setText = (text) => {
    item.text(text);
  }

  return {
    setActive,
    setText
  }
}

function addOverlayClickedCallback() {
  activeOverlay = serviceArrTmp[activeItem].overlayList.length;
  added = true;
  const newOverlay = serviceArrTmp[activeItem].overlayList.length > 0 ? getLastOverlay() : getDefaultOverlay();
  serviceArrTmp[activeItem].overlayList.push(newOverlay);
  setEditedOverlay();
  $('#editOverlayModal').modal('show');
}

function getLastOverlay() {
  const lastItem = serviceArrTmp[activeItem].overlayList.length - 1;
  const itemToCopy = Object.create(serviceArrTmp[activeItem].overlayList[lastItem]);
  itemToCopy.active = false;
  return itemToCopy;
}

function getDefaultOverlay() {
  return {
    "active": false,
    "coordSystem": "top_left",
    "pos_x": 10,
    "pos_y": 10,
    "imgPath": "",
    "imgName": "",
    "duration": 5,
    "scale": 1.0
  };
}

async function setEditedOverlay() {
  createPositionList('#customPosition', serviceArrTmp[activeItem].overlayList[activeOverlay].coordSystem);
  $('#customPosX').val(serviceArrTmp[activeItem].overlayList[activeOverlay].pos_x);
  $('#customPosY').val(serviceArrTmp[activeItem].overlayList[activeOverlay].pos_y);
  let customScale = convertFloatToPercentage(serviceArrTmp[activeItem].overlayList[activeOverlay].scale);
  if (isNaN(customScale)) {
    customScale = 100;
  }
  $('#customScale').val(customScale);
  $('#customDuration').val(serviceArrTmp[activeItem].overlayList[activeOverlay].duration);
  if (!serviceArrTmp[activeItem].overlayList[activeOverlay].fps) {
    $('#customFPS').val(1);
  } else {
    $('#customFPS').val(serviceArrTmp[activeItem].overlayList[activeOverlay].fps);
  }


  // Create image list field
  await createImageList('#customImgPath', true, serviceArrTmp[activeItem].overlayList[activeOverlay].imgPath);
  customImagePathChanged();
}

function customImagePathChanged() {
  let options = $('#customImgPath option');
  let selectedIndex = $('#customImgPath')[0].selectedIndex;
  if (options.length === 0) {
    $('#customPreviewEditOverlay').html('');
    return;
  }
  let fileSrc = getImageSrc(options[selectedIndex].getAttribute('value'));
  if (!fileSrc.includes('http')) {
    $('#editOverlayModal #customFPS').prop('disabled', true);
    $('#editOverlayModal .editImgFPS').hide();
  } else {
    $('#editOverlayModal #customFPS').prop('disabled', false);
    $('#editOverlayModal .editImgFPS').show();
  }
  $('#imagePreviewEditOverlay').html('<img id="imageManagerPreview" src="' + fileSrc + '" />');
}

function getImageSrc(fileSrc) {
  return '/local/camoverlay/api/image.cgi?path=' + encodeURIComponent(fileSrc) + '&_t=' + Date.now();
}

async function saveEditedOverlayCallback() {
  added = false;
  let imageList = await imageManager.getFileList();
  var image = findImageByPath(imageList, $('#customImgPath').val());
  serviceArrTmp[activeItem].overlayList[activeOverlay].coordSystem = $('#customPosition').val();
  serviceArrTmp[activeItem].overlayList[activeOverlay].pos_x = parseInt($('#customPosX').val());
  serviceArrTmp[activeItem].overlayList[activeOverlay].pos_y = parseInt($('#customPosY').val());
  serviceArrTmp[activeItem].overlayList[activeOverlay].imgPath = image.path;
  serviceArrTmp[activeItem].overlayList[activeOverlay].imgName = image.name;
  serviceArrTmp[activeItem].overlayList[activeOverlay].duration = parseInt($('#customDuration').val());
  if (!$('#customFPS').prop('disabled')) {
    serviceArrTmp[activeItem].overlayList[activeOverlay].fps = parseInt($('#customFPS').val())
  } else if (serviceArrTmp[activeItem].overlayList[activeOverlay].fps) {
    delete serviceArrTmp[activeItem].overlayList[activeOverlay].fps;
  }
  serviceArrTmp[activeItem].overlayList[activeOverlay].scale = convertPercentageToFloat($('#customScale').val(), 100);
  showOverlayList();
}

function cancelEditedOverlayCallback() {
  if (added) {
    added = false;
    serviceArrTmp[activeItem].overlayList.pop();
  } else {
    serviceArrTmp[activeItem].overlayList[activeOverlay] = savedOverlay;
  }
  showOverlayList();
}

function moveOverlayClicked(event) {
  var targ = event.target || event.srcElement;
  var linkElement = $(targ).closest('a');
  var index = linkElement.attr('id').substring(11);
  var newIndex = parseInt(index) + 1;
  if (linkElement.hasClass('overlayMoveUp'))
    newIndex = parseInt(index) - 1;
  if (newIndex < 0 || newIndex >= serviceArrTmp[activeItem].overlayList.length)
    return false;
  var tmp = serviceArrTmp[activeItem].overlayList[index];
  serviceArrTmp[activeItem].overlayList[index] = serviceArrTmp[activeItem].overlayList[newIndex];
  serviceArrTmp[activeItem].overlayList[newIndex] = tmp;
  showOverlayList();
  return false;
}

function editOverlayClicked(event) {
  var targ = event.target || event.srcElement;
  var index = $(targ).closest('a').attr('id').substring(11);
  activeOverlay = index;
  savedOverlay = serviceArrTmp[activeItem].overlayList[activeOverlay];
  setEditedOverlay();

  if (!$(targ).closest('a').hasClass('show-fps')) {
    $('#editOverlayModal #customFPS').prop('disabled', true);
    $('#editOverlayModal .editImgFPS').hide();
  } else {
    $('#editOverlayModal #customFPS').prop('disabled', false);
    $('#editOverlayModal .editImgFPS').show();
  }

  $('#editOverlayModal').modal('show');
  return false;
}

function copyOverlayClicked(event) {
  const targ = event.target || event.srcElement;
  const index = $(targ).closest('a').data('id');
  const overlayToCopy = serviceArrTmp[activeItem].overlayList[index];
  const newOverlay = JSON.parse(JSON.stringify(overlayToCopy));
  newOverlay.active = false;
  serviceArrTmp[activeItem].overlayList.push(newOverlay);
  serviceArr[activeItem].overlayList = serviceArrTmp[activeItem].overlayList;
  showOverlayList();
  return false;
}

function removeOverlayClicked(event) {
  var targ = event.target || event.srcElement;
  var index = $(targ).closest('a').attr('id').substring(13);
  serviceArrTmp[activeItem].overlayList.splice(index, 1);
  serviceArr[activeItem].overlayList = serviceArrTmp[activeItem].overlayList;
  showOverlayList();
  return false;
}