function Capture() {
  let capture = null;

  let isWebcamSupported = () => {
    if (navigator.mediaDevices.getUserMedia) {
      return true;
    }
    return false;
  }

  let isScreenSharingSupported = () => {
    if (navigator.mediaDevices.getDisplayMedia) {
      return true;
    }
    return false;
  }

  let handleSharingErrors = (errName, isWebcam) => {
    if (errName === 'NotFunctionError') {
      return 'Not possible to share a mobile screen.';
    } else if (errName === 'NotAllowedError') {
      if (isWebcam) {
        return 'Allow access to your computer camera.';
      }
      return 'Allow access to your computer screen.';
    } else if (errName === 'NotFoundError') {
      return 'No source available.';
    } else if (errName === 'NotReadableError') {
      return 'Not readable source.';
    } else if (errName === 'SecurityError') {
      return 'Disabled in your browser.';
    }
    return 'Unable to start.';
  }

  let setScreen = async (fps) => {
    if (!isScreenSharingSupported()) {
      throw Error(handleSharingErrors('NotFunctionError', false));
    }

    try {
      capture = await navigator.mediaDevices.getDisplayMedia({ video: { frameRate: fps } });
    } catch (err) {
      console.error(err);
      throw Error(handleSharingErrors(err.name, false));
    }
    return capture;
  }

  let setWebcam = async (fps, deviceId) => {
    if (!isWebcamSupported()) {
      throw Error(handleSharingErrors('NotFunctionError', true));
    }

    const videoConfig = { frameRate: fps, deviceId: deviceId };

    try {
      capture =  await navigator.mediaDevices.getUserMedia({
        audio: false,
        video: videoConfig
      });
    } catch (err) {
      console.error(err);
      throw Error(handleSharingErrors(err.name, true));
    }
    return capture;
  }

  let stop = () => {
    if (capture != undefined) {
      capture.getVideoTracks().forEach(item => {
        item.enabled && item.stop();
      });
    }
  }

  let onEnded = (callback) => $(capture.getVideoTracks()).off('ended').on('ended', () => callback());
  let set = (mediaStream) => capture = mediaStream;
  let get = () => capture;
  let getSettings = () => capture.getVideoTracks()[0].getSettings();
  let handleError = (constraintName) => {
    console.error("Constraint " + constraintName + " is not supported");
    throw Error('Unable to change '+ constraintName);
  }

  let setFps = async (fps) => {
    let supports = navigator.mediaDevices.getSupportedConstraints();
    if (!supports["frameRate"]) {
      handleError("frameRate");
    }
    await capture.getVideoTracks()[0].applyConstraints({ frameRate: fps });
  }

  let getWebcamList = async () => {
    if (!navigator.mediaDevices || !navigator.mediaDevices.enumerateDevices) {
      throw Error("Can't reach any camera on your computer");
      return;
    }

    let deviceList = null;
    try {
      deviceList = await navigator.mediaDevices.enumerateDevices();
    } catch(err) {
      throw Error("Can't find any web camera on your computer");
      console.error(err.name + ": " + err.message);
      return;
    }

    return deviceList.filter(device => device.kind === "videoinput");
  }

  return {
    set,
    setScreen,
    setWebcam,
    setFps,
    stop,
    onEnded,
    get,
    getWebcamList,
    getSettings,
  }
}