function PtzManager() {

  let getList = async function (cameraChannel) {
    let result;
    try {
      result = await vapixApi.getPtzData({
        'query': 'presetposcam',
        'camera': ++cameraChannel,
      });
    } catch (error) {
      throw new Error(error.message);
    }

    const posNameList = result.split('\n');

    const ptzList = [];
    for (var i = 1; i < posNameList.length - 1; i++) {
      var presetName = posNameList[i].split('=');
      ptzList.push(presetName[1].trim());
    }

    return ptzList;
  }

  let getListData = async function (cameraChannel) {
    let result;
    try {
      result = await vapixApi.getPtzData({
        'query': 'presetposcamdata',
        'camera': ++cameraChannel,
      });
    } catch (error) {
      throw new Error(error.message);
    }

    result = result.split('\n');
    result.shift(); // Remove description of preset
    result.pop(); // Remove empty row on the end

    resultArr = [];
    result.forEach((position, index) => {
      let positions = position.split(':');
      let ptz_param = { name: null, pan: null, tilt: null, zoom: null };

      positions.forEach((position_param, i) => {
        position_param = position_param.split('=');

        if (position_param[0].indexOf('presetposno') > -1) {
          ptz_param.name = position_param[1];
        }
        if (position_param[0] === 'pan') {
          ptz_param.pan = parseFloat(position_param[1]).toFixed(2);
        }
        if (position_param[0] === 'tilt') {
          ptz_param.tilt = parseFloat(position_param[1]).toFixed(2);
        }
        if (position_param[0] === 'zoom') {
          ptz_param.zoom = parseInt(position_param[1]).toString();
        }
      });

      resultArr[index] = ptz_param;
    });

    return resultArr;
  }

  let getPosition = async function () {
    try {
      return await vapixApi.getParsedPtzData({
        'query': 'position',
      });

    } catch (error) {
      throw new Error(error.message);
    }
  }

  let getCurrentPosition = async function () {
    let positionData = await getListData();
    let current = await getPosition();
    let result = '';
    positionData.forEach(item => {
      let distance = 0.02;
      let distanceZoom = 10;
      if (Math.abs(item.pan - current.pan) <= distance && Math.abs(item.tilt - current.tilt) <= distance && Math.abs(item.zoom - current.zoom) <= distanceZoom) {
        result = item.name;
      }
    });
    return result;
  }

  let setPosition = async function (presetName, cameraChannel) {
    try {
      await vapixApi.goToPtzPosition({
        'gotoserverpresetname': presetName.trim(),
        'camera': ++cameraChannel,
      });

    } catch (error) {
      throw new Error(error.message);
    }
  }

  return {
    getList,
    getCurrentPosition,
    getPosition,
    setPosition
  }

}