function VideoElement(serviceId, container) {
  let videoEl;
  let videoId;

  let get = () => videoEl;

  let create = async (mediaStream) => {
    return new Promise((resolve) => {
      const videoHTML = $('<video class="videoSourceElement img-fluid rounded" autoplay></video>');
      const videoContainerHTML = $(`
        <div class="container_videoSourceElement">
          <div class="videoSourceName">Service ID: ` + serviceId + `</div>  
          <div class="videoSourceContainerBtns">
            <button id="videoSourceBtnClose` + serviceId + `" class="videoSourceBtnClose btn btn-sm" title="Hide Preview">
              <div class="fa-1x">
                <span class="fa-layers fa-fw">
                  <i class="fas fa-slash fa-rotate-90"></i>
                  <i class="fas fa-eye" data-fa-transform="shrink-2"></i>
                </span>
              </div>
            </button>
            <button class="videoSourceBtnStop btn btn-sm btn-danger" title="Stop Sharing" data-video-id="#videoSource` + serviceId + `">
                <div class="fa-1x">
                  <span class="fa-layers fa-fw">
                    <i class="fas fa-slash fa-rotate-90"></i>
                    <i class="fas fa-desktop" data-fa-transform="shrink-2"></i>
                  </span>
                </div>
            </button>
          </div>
        </div>
      `);
      videoId = 'videoSource' + serviceId;
      containerVideoId = 'container_'+videoId;
      videoHTML.attr('id', videoId);
      videoContainerHTML.attr('id', containerVideoId);
      if (existsInContainer(container, containerVideoId)) {
        appendToContainer(container, videoContainerHTML);
        if (existsInContainer(videoContainerHTML, videoId)) {
          appendToContainer(videoContainerHTML, videoHTML);
        }
      }
      videoEl = $('#' + videoId)[0];
      videoEl.srcObject = mediaStream;
      resolve();
    });
  }
  
  let existsInContainer = (container, videoId) => container.find('#' + videoId).length === 0;
  let appendToContainer = (container, htmlElement)  => container.append(htmlElement);
  let onLoadedData = (callback) => $(videoEl).off('loadeddata').on('loadeddata', callback);
  let onStop = (serviceId, callback) => $('.videoSourceBtnStop[data-video-id="#videoSource' + serviceId + '"]').click(callback);
  let onHidePreview = (callback) => {
    $('#videoSourceBtnClose' + serviceId).click(function() {
      $(this).closest('.container_videoSourceElement').hide();
      callback(serviceId);
    });
  }

  let isPreviewVisible = async () => $(videoEl).closest('.container_videoSourceElement').is(":visible");
  let showPreview = () => { $(videoEl).closest('.container_videoSourceElement').show(); }
  let remove = () => { $(videoEl).closest('.container_videoSourceElement').remove(); }
  let play = () => { videoEl.play(); }
  let pause = () => { videoEl.pause(); }
  let stop = () => { videoEl.stop(); }

  return {
    get,
    create,
    play,
    pause,
    stop,
    remove,
    isPreviewVisible,
    showPreview,
    onLoadedData,
    onStop,
    onHidePreview,
  }
}