class ServerApi {
    constructor(requestManager) {
        this.requestManager = requestManager;
    }

    async sendStatistics(serverUrl, serialNumber, params) {
        try {
            await this.requestManager.postFormData(
                `${serverUrl}/index.php?p=CamOverlay&a=updateServices&serial_number=${serialNumber}`,
                params
            );
        } catch (e) {
            console.error("Error occured while trying to send statistics.");
        }

    }

    async getMapData(serverUrl, serialNumber, locationKey, lang) {
        let response;
        try {
            response = await this.requestManager.get(
                `${serverUrl}/index.php?p=CamOverlay&a=AccuWeatherProxy&serial_number=${serialNumber}&q=${encodeURIComponent(`/locations/v1/${encodeURIComponent(locationKey)}?language=${encodeURIComponent(lang)}`)}`
            );

        } catch (error) {
            console.error(`${error.name}: ${error.message}`);
        }

        if (!response.ok) {
            console.error('Failed to fetch map data')
        }

        return await this.requestManager.handleParseJsonResponseError(response);
    }

    async getMapLocation(serverUrl, serialNumber, query, lang) {
        let response;
        try {
            response = await this.requestManager.get(`${serverUrl}/index.php?p=CamOverlay&a=AccuWeatherProxy&serial_number=${serialNumber}&q=${encodeURIComponent(`/locations/v1/cities/geoposition/search?q=${encodeURIComponent(query)}&language=${encodeURIComponent(lang)}`)}`);

        } catch (error) {
            console.error(`${error.name}: ${error.message}`);
        }

        if (!response.ok) {
            console.error('Failed to get map location')
        }

        return await this.requestManager.handleParseJsonResponseError(response);
    }

    async getSuggestedMapLocation(serverUrl, serialNumber, query, lang) {
        let response;
        try {
            response = await this.requestManager.get(`${serverUrl}/index.php?p=CamOverlay&a=AccuWeatherProxy&serial_number=${serialNumber}&q=${encodeURIComponent(`/locations/v1/cities/autocomplete?q=${encodeURIComponent(query)}&language=${encodeURIComponent(lang)}`)}`);

        } catch (error) {
            console.error(`${error.name}: ${error.message}`);
        }

        if (!response.ok) {
            console.error('Failed to get suggested map location')
        }

        return await this.requestManager.handleParseJsonResponseError(response);
    }

    async handleLicenses(url, params) {
        let response;
        try {
            response = await this.requestManager.get(`${url}?${this.requestManager.buildQuery(params)}`);

        } catch (error) {
            console.error(`${error.name}: ${error.message}`);
        }

        return await this.requestManager.handleParseJsonResponseError(response);
    }
}