
var alertID = 0;
var propertiesResponse = null;

function getImgSize(imgSrc) {
  var promise = new Promise(function (resolve, reject) {
    var newImg = new Image();

    newImg.onload = function () {
      resolve({ width: newImg.width, height: newImg.height });
    }

    newImg.src = imgSrc; // this must be done AFTER setting onload
  });
  return promise;
}

const renderLayerZindex = (container) => {
  container.replaceWith($(`
    <div class="input-group mt-2" data-toggle="tooltip"
       title="Specifies the order of the layers, the higher number is shown at the top.">
    <label htmlFor="layerZindex" class="col-lg-4 control-label">Layer order</label>
    <div class="col-lg-8">
      <select class="form-control" id="layerZindex">
        <option value="0">0</option>
        <option value="1">1</option>
        <option value="2">2</option>
        <option value="3">3</option>
        <option value="4">4</option>
        <option value="5">5</option>
        <option value="6">6</option>
        <option value="7">7</option>
        <option value="8">8</option>
        <option value="9">9</option>
        <option value="10">10</option>
      </select>
    </div>
  </div>`));
}

const initLayerZindex = async (value) => {
  renderLayerZindex($('#layerZindexPlaceholder'));
  const layerZindex = $('#layerZindex');
  const isVisible = await isArmv7hfOrHigher();
  layerZindex.prop('disabled', !isVisible);
  if (isVisible) {
    layerZindex.val(value);
    return;
  }
  layerZindex.closest('.input-group').hide();
}

const saveLayerZindex = async (value) => {
  if (await isArmv7hfOrHigher()) {
    serviceArrTmp[activeItem].zIndex = parseInt(value);
  }
}

const getSupportedVersion = async () => {
  if (!cameraParams.getSupportedVersion()) {
    const result = await vapixApi.getBasicDeviceInfo({
      'method': 'getSupportedVersions'
    });
    cameraParams.setSupportedVersion(result.apiVersions[0]);
  }
  return cameraParams.getSupportedVersion();
}

const getSoc = async () => {
  if (!cameraParams.getSoc()) {
    const supportedVersion = await getSupportedVersion();
    const result = await vapixApi.getBasicDeviceInfo({
      'apiVersion': supportedVersion,
      'method': 'getAllProperties'
    });
    cameraParams.setSoc(result.propertyList.Soc);
  }
  return cameraParams.getSoc();
}

const isArmv7hfOrHigher = async () => {
  const soc = await getSoc();
  if (typeof soc === 'undefined') {
    throw new Error('Unknown Soc parameter');
  }
  return !soc.includes('Artpec-6');
}

function firstToUpperCase(str) {
  return str.substr(0, 1).toUpperCase() + str.substr(1);
}

areArrayContentsEqual = (arr1, arr2) => {
  if (arr1.length !== arr2.length) return false;
  for (let i = 0; i < arr1.length; i++) {
    if (!arr2.includes(arr1[i])) return false;
  }
  return true;
}

hasPtzOverlayServiceChanged = async () => {
  const servicesCopy = JSON.parse(JSON.stringify(serviceArr[activeItem]));
  const tmpServicesCopy = JSON.parse(JSON.stringify(serviceArrTmp[activeItem]));

  // exclude ptz positions
  savedPtzPositionsList = servicesCopy.ptz_positions;
  tmpPtzPositionsList = tmpServicesCopy.ptz_positions;

  delete servicesCopy.ptz_positions;
  delete tmpServicesCopy.ptz_positions;

  if (JSON.stringify(servicesCopy) !== JSON.stringify(tmpServicesCopy)) return true;

  // check positions

  let ptzList;
  try {
    ptzList = await ptzManager.getList(tmpServicesCopy.cameraList[0]);
  } catch (e) {
    console.error('Could not get ptzlist');
    return;
  }

  const contentsEqual = areArrayContentsEqual(Object.keys(savedPtzPositionsList), ptzList);
  if (!contentsEqual) return true;

  Object.keys(tmpPtzPositionsList).forEach((ptzPosName) => {
    if (!ptzList.includes(ptzPosName)) {
      delete tmpPtzPositionsList[ptzPosName];
    }
  })

  return JSON.stringify(savedPtzPositionsList) !== JSON.stringify(tmpPtzPositionsList)
}

async function showSaveWarning() {
  let shouldShowWarning = JSON.stringify(serviceArr[activeItem]) !== JSON.stringify(serviceArrTmp[activeItem]);
  // ugly hack for ptz positions in ptz overlay service
  if (serviceArrTmp[activeItem].name === 'ptz') {
    shouldShowWarning = await hasPtzOverlayServiceChanged();
  }

  if (shouldShowWarning) {
    const warningAlertName = 'saveWarning';
    const warningText = 'Unsaved changes';
    let html = '<div id="' + warningAlertName + '" class="alert alert-warning d-inline-flex" data-toggle="tooltip" title="' + warningText + '">';
    html += '<strong><span class="fas fa-exclamation-triangle"></span></strong><span class="ml-2 d-none d-sm-block">' + warningText + '</span>';
    html += '</div>';
    $('#warningArea').html(html);
    if ($(window).width() < 769) { // Display tooltip only on mobile
      $('#' + warningAlertName).tooltip();
    }
  } else {
    hideSaveWarning();
  }
}

function validate(element, invalidValue, invalidMsg) {
  var validation = (element.val() === invalidValue) ? 'is-invalid' : '';
  var invalidDiv = '<div class="invalid-feedback">' + invalidMsg + '</div>';

  if (validation === '') {
    element.removeClass('is-invalid');
    element.closest('.input-group').find('.invalid-feedback').remove();
  } else {
    element.addClass(validation);
    element.closest('.input-group').find('.invalid-feedback').remove();
    element.closest('.input-group').append(invalidDiv);
  }
}

function disableSaveBtn() {
  $('#saveBtn').prop('disabled', true);
}

function enableSaveBtn() {
  $('#saveBtn').prop('disabled', false);
}

function configExport() {
  $('.downloadConfiguration').off('click').on('click', function (e) {
    e.preventDefault();
    var data = "text/json;charset=utf-8," + encodeURIComponent(JSON.stringify(serviceArrTmp[activeItem]));
    var a = document.createElement('a');
    a.href = 'data:' + data;
    a.download = 'exportConfiguration-' + new Date().getTime().toString() + '.json';
    a.click();
  });
}

function configImport() {
  const configUploadInput = $('#configurationFileUpload');
  $('.uploadConfiguration').off('click').on('click', () => {
    configUploadInput.trigger('click');
  });

  configUploadInput.off('change').on('change', () => {
    let files = document.getElementById('configurationFileUpload').files;
    if (files.length <= 0) {
      showMessage('warning', 'Empty file input.', false, 'emptyFileInput');
    }
    let fr = new FileReader();
    fr.onload = (e) => {
      try {
        let importedData = JSON.parse(e.target.result);
        importedData.id = serviceArrTmp[activeItem].id;
        serviceArrTmp[activeItem] = importedData;
        $('[data-toggle="tooltip"], .tooltip').tooltip('dispose'); //destroy tooltips from previous page
        createServiceList(serviceArrTmp);
        setServiceItemActive(activeItem);
        showMessage('success', 'Configuration was successfully imported', false, 'configurationUploadSuccess');
      } catch (e) {
        showMessage('danger', 'JSON parsing goes wrong.', false, 'jsonParseError');
      } finally {
        configUploadInput.val('');
      }
    };
    if (files.item(0).type !== "application/json") {
      showMessage('danger', 'This file is not allowed.', false, 'wrongFileType');
      configUploadInput.val('');
    }
    fr.readAsText(files.item(0));
    return false;
  });
}

$.fn.showSpinner = function (text = '') {
  var loadingDiv = '<div class="loadingDiv align-items-center align-items-center text-center row rounded h-100" style="position:absolute;top:0;bottom:0;left:15px;right:15px;background-color: #f7f7f7;z-index: 1020;">';
  loadingDiv += '<div class="col"><p class="text-primary fas fa-2x fa-spinner fa-pulse"></p>';
  if (text !== '') loadingDiv += '<p class="text-muted mb-0">' + text + '</p>';
  loadingDiv += '</div></div>';
  $(this).append(loadingDiv);
};

$.fn.hideSpinner = function () {
  $(this).find('.loadingDiv').remove();
};

function hideSaveWarning() {
  $('#warningArea').html('');
}

function createResolutionList(selectID) {
  var selected = 0;
  var cameraResolution = -1;
  $(selectID).empty();
  var resItem = $(selectID)[0];
  for (let i = 0; i < cameraResolutions.length; i++) {
    if (cameraResolutions[i].width === serviceArrTmp[activeItem].width && cameraResolutions[i].height === serviceArrTmp[activeItem].height) {
      selected = i;
    }
    if (cameraResolutions[i].width === resolution.width && cameraResolutions[i].height === resolution.height) {
      cameraResolution = i;
    }
    resItem.options[resItem.options.length] = new Option(cameraResolutions[i].width + ' x ' + cameraResolutions[i].height, i);
  }
  resItem.selectedIndex = selected;
  if (cameraResolution != -1) {
    $(selectID + ' option[value="' + cameraResolution + '"]').css('background-color', '#B6DAFF');
  }
}

async function imageExists(fileSrc) {
  try {
    await requestManager.get(fileSrc);
    return true;

  } catch (error) {
    return false;
  }
}

function createAddonsContainer() {
  return `<span class="hw_badge hw_badge_on btn btn-sm btn-success">Input is ON</span>
          <span class="hw_badge hw_badge_off btn btn-sm btn-danger">Input is OFF</span>
          <button type="button" class="btn btn-sm btn-primary" id="scheduleBtn" data-toggle="tooltip"
                  title="This function allows you to configure times for displaying this widget.">
            <span class="fa fa-calendar-alt mr-1"></span>
            Set schedule
          </button>`;
}

async function createTriggeringElement(identificator, selectedVal, scheduleVal, automaticValueAllowed) {
  const containerDiv = $(identificator);
  const addonsContainer = createAddonsContainer();
  containerDiv.next('.addonsContainer').html(addonsContainer);
  let radioButtonList = [];
  if (automaticValueAllowed) {
    radioButtonList.push({ label: 'Automatic', name: 'automationType', value: 'time' });
  }
  radioButtonList.push({ label: 'Manual', name: 'automationType', value: 'manual' });
  radioButtonList.push({ label: 'Schedule', name: 'automationType', value: 'schedule' });

  // Check if schedule is empty - set manual triggering, otherwise set schedule triggering
  if (selectedVal === undefined) {
    selectedVal = "schedule";
    if (scheduleVal === "") {
      selectedVal = "manual";
    }
  }

  let nbrOfInputs
  try {
    nbrOfInputs = parseInt(cameraParams.get('Input.NbrOfInputs') || await vapixApi.getParameter('Input.NbrOfInputs'));
  } catch (error) {
    throw new Error(error.message);
  }

  let params = {};
  const paramsToRequest = [];

  for (let i = 0; i < nbrOfInputs; i++) {
    cameraParams.handleUnknownParams(`IOPort.I${i}.Direction`, params, paramsToRequest);
    cameraParams.handleUnknownParams(`IOPort.I${i}.Input.Name`, params, paramsToRequest);
  }

  let requestedParams;
  if (paramsToRequest.length > 0) {
    try {
      requestedParams = await vapixApi.getParameters(paramsToRequest);
    } catch (error) {
      throw new Error(error.message);
    }
  }

  params = {
    ...params,
    ...requestedParams,
  }

  for (let i = 0; i < nbrOfInputs; i++) {
    if (params[`IOPort.I${i}.Direction`] === 'input') {
      let io_port_name = params[`IOPort.I${i}.Input.Name`].replaceAll(' ', '&nbsp;');
      radioButtonList.push({ label: 'I/O port (' + io_port_name + ')', name: 'automationType', value: 'input' + i });
    }
  }

  if (selectedVal === undefined) {
    showMessage('warning', 'Trigger option' + (selectedVal === undefined ? ' ' : selectedVal) + 'is not valid. Reseting to default trigger.', false, 'triggerValueNotFound');
    selectedVal = 'manual';
  }

  const triggeringRadioButtons = new RadioButtons();
  triggeringRadioButtons.create(identificator, true, true, radioButtonList, selectedVal);

  // Change event
  containerDiv.find('[name="automationType"]').each(function () {
    let item = $(this);
    item.change(function () {
      checkHwIoState(item);
      toggleScheduleBtn(item);
    });
  });

  let checkedRadioElement = containerDiv.find('[name="automationType"][value=' + selectedVal + ']');
  checkedRadioElement.change();
}

function isServiceEnabled(serviceId) {
  const enableToggle = $('.serviceListItem[data-id="' + serviceId + '"]').find('.switch')[0];
  return parseBool(enableToggle.getAttribute('checked'));
}

function toggleScheduleBtn(triggeringElement) {
  let isSchedule = triggeringElement.val().match(/schedule/) !== null;
  $('#scheduleBtn').toggle(isSchedule);
}

async function checkHwIoState(triggeringElement) {
  let isHwInput = triggeringElement.val().match(/input\d+/) !== null;

  if (typeof timeOut === 'number') {
    clearTimeout(timeOut);
  }

  if (isHwInput) {
    let inputNumber = parseInt(triggeringElement.val().replace('input', '')) + 1;
    let response;
    try {
      response = await vapixApi.getIOPortParameter({
        'checkactive': inputNumber,
      });
    } catch (error) {
      throw new Error(error.message);
    }

    if (response['port' + inputNumber] === "active") {
      $('.hw_badge.hw_badge_off').hide();
      $('.hw_badge.hw_badge_on').show();
    } else {
      $('.hw_badge.hw_badge_on').hide();
      $('.hw_badge.hw_badge_off').show();
    }
    timeOut = setTimeout(checkHwIoState, 5000, triggeringElement);

  } else {
    $('.hw_badge').hide();
  }
}

async function getVideoChannelDataList() {
  let configCount;
  try {
    configCount = parseInt(cameraParams.get('Image.NbrOfConfigs') || await vapixApi.getParameter('Image.NbrOfConfigs'));

  } catch (error) {
    throw new Error(error.message);
  }

  let params = {};
  const paramsToRequest = [];

  cameraParams.handleUnknownParams('Properties.Image.Resolution', params, paramsToRequest);

  for (let i = 0; i < configCount; i++) {
    cameraParams.handleUnknownParams(`Properties.Image.I${i}.Resolution`, params, paramsToRequest);
    cameraParams.handleUnknownParams(`Image.I${i}.Appearance.Resolution`, params, paramsToRequest);
    cameraParams.handleUnknownParams(`Image.I${i}.Name`, params, paramsToRequest);
    cameraParams.handleUnknownParams(`Image.I${i}.Source`, params, paramsToRequest);
    cameraParams.handleUnknownParams(`Image.I${i}.Enabled`, params, paramsToRequest);
  }

  let requestedParams;
  if (paramsToRequest.length > 0) {
    try {
      requestedParams = await vapixApi.getParameters(paramsToRequest);
    } catch (error) {
      throw new Error(error.message);
    }
  }

  params = {
    ...params,
    ...requestedParams,
  }

  return {
    channelCount: configCount,
    params
  };
}

async function createVideoChannelElement(selectID, selectedValues, selectMultiple = true) {
  if (selectedValues[0] === -1) selectedValues = [];

  const vcData = await getVideoChannelDataList();

  createVideoChannelList(selectID, selectedValues, vcData.channelCount, vcData.params, selectMultiple);

  $(selectID).change();
}


function getValuesFromSelect(selectValue) {
  if (!selectValue) return [-1]; // nothing selected
  return [parseInt(selectValue)];
}

function getValuesFromSelectMultiple(selectValues) {
  if (!selectValues || selectValues.length === 0) return [-1]; // nothing selected
  return selectValues.map((strValue) => parseInt(strValue));
}

function getSelectedVideoChannelValues(element) {
  const selectValue = element.val();
  if (Array.isArray(selectValue)) {
    return getValuesFromSelectMultiple(selectValue);
  }

  return getValuesFromSelect(selectValue);
}

function onCustomNameKeyUp(e) {
  let value = $(e.target).val();
  value = escapeHtml(value);
  serviceArrTmp[activeItem].customName = value;
  showSaveWarning();
  const serviceNameElement = $('.serviceListItemActive > .serviceName');
  serviceNameElement.html(
    (value === '') ? serviceName[serviceArrTmp[activeItem].name] : '<span>' + value + '</span>'
  );
}

/**
 *
 */
function onResolutionChange(resolutionId, videoChannelId) {
  $(resolutionId).change(function () {
    let currentCameraResolutionObject = cameraResolutions[parseInt($(resolutionId).val())];
    const selectedCamera = serviceArrTmp[activeItem].cameraList.length > 0 ? serviceArrTmp[activeItem].cameraList[0] : 0;
    showImageFromCamera('imagePreviewCamera', currentCameraResolutionObject, selectedCamera);
  });
}

/**
 *
 * @param {string} resolutionId - id of select element for resolution
 * @param {string} videoChannelId - id of select element for video channel
 */
function onVideoChannelChange(resolutionId, videoChannelId, videoPreviewSelectId) {
  $(videoChannelId).change(async function () {
    let selectedCameraList = getSelectedVideoChannelValues($(this));
    selectedCamera = selectedCameraList[0] === -1 ? 0 : selectedCameraList[0]; // by default select first camera

    try {
      cameraResolutions = createNewResolutionList(
        cameraParams.getCameraResolutionsForCamera(selectedCamera)
        || await vapixApi.getParameter(cameraParams.dynamicParamKeys['cameraResolutions'](selectedCamera))
        || cameraParams.getCameraResolutions()
      );

      resolution = createResolutionObject(
        cameraParams.getCameraResolution(selectedCamera)
        || await vapixApi.getParameter(cameraParams.dynamicParamKeys['cameraResolution'](selectedCamera))
      );
    } catch (error) {
      throw new Error(error.message);
    }

    createResolutionList(resolutionId);
    formChangedCallback();

    // populate preview image video channel select
    await createVideoChannelElement(videoPreviewSelectId, selectedCameraList, false);

    let currentCameraResolutionObject = cameraResolutions[parseInt($(resolutionId).val())];
    showImageFromCamera('imagePreviewCamera', currentCameraResolutionObject, selectedCamera);
  });
}

function onPreviewVideoChannelChange(elementId, resolutionId) {
  $(elementId).change(function () {
    let currentCameraResolutionObject = cameraResolutions[parseInt($(resolutionId).val())];
    showImageFromCamera('imagePreviewCamera', currentCameraResolutionObject, parseInt($(elementId).val()));
  });
}

// works also for ordinary select, but selects the last selected
function createVideoChannelList(selectID, selectedValues, videoChannelCount, cameraProperties, selectMultiple) {
  var element = $(selectID);
  element.empty();
  const selectedValuesTmp = [];
  for (var i = 0; i < videoChannelCount; i++) {
    if (cameraProperties[`Image.I${i}.Enabled`] === 'yes' || i === 0) {
      const optionLabel = cameraProperties[`Image.I${i}.Name`];
      element.append(`<option value="${i}">${optionLabel}</option>`);
      if (selectedValues.length > 0 && selectedValues.includes(i)) {
        selectedValuesTmp.push(i);
      }
    }
  }
  if (selectMultiple) {
    element.val(selectedValuesTmp.length > 0 ? selectedValuesTmp : []);
    element.selectpicker("refresh");
  } else {
    element.val(selectedValuesTmp.length > 0 ? selectedValuesTmp[0] : 0);
  }
}

/**
 * Help to set default resolution for video channel
 * @param {string} resolutionProperty
 * @returns {{width: number, height: number}}
 */
function createResolutionObject(resolutionProperty) {
  const res = resolutionProperty.split('x');
  if (res.length > 1) {
    return {
      width: parseInt(res[0].trim()),
      height: parseInt(res[1].trim())
    }
  }
}

/**
 * Help to create new resolution
 * @param {string} resolutionProperties
 * @returns {this}
 */
function createNewResolutionList(resolutionProperties) {
  let resolutionList = getAvailableResolution(resolutionProperties);
  return resolutionList.sort(sortFunc);
}

function getAvailableResolution(params) {
  let cameraResolutions = [];
  let resolutionsStr = params.split(',');
  for (let i = 0; i < resolutionsStr.length; i++) {
    let resObject = createResolutionObject(resolutionsStr[i]);
    cameraResolutions.push(resObject);
  }
  return cameraResolutions;
}

function createPositionList(selectID, selectedVal) {
  $(selectID).html('');
  var posItem = $(selectID);
  posItem.empty();
  posItem.append(new Option('Top Left', 'top_left'));
  posItem.append(new Option('Top Center', 'top'));
  posItem.append(new Option('Top Right', 'top_right'));
  posItem.append(new Option('Left Center', 'left'));
  posItem.append(new Option('Center', 'center'));
  posItem.append(new Option('Right Center', 'right'));
  posItem.append(new Option('Bottom Left', 'bottom_left'));
  posItem.append(new Option('Bottom Center', 'bottom'));
  posItem.append(new Option('Bottom Right', 'bottom_right'));
  posItem.val(selectedVal);
}

function coordSystemToNiceName(coordSystem) {
  if (coordSystem == 'top_left')
    return 'Top Left';
  else if (coordSystem == 'top')
    return 'Top Center';
  else if (coordSystem == 'top_right')
    return 'Top Right';
  else if (coordSystem == 'left')
    return 'Left Center';
  else if (coordSystem == 'center')
    return 'Center';
  else if (coordSystem == 'right')
    return 'Right Center';
  else if (coordSystem == 'bottom_left')
    return 'Bottom Left';
  else if (coordSystem == 'bottom')
    return 'Bottom Center';
  else if (coordSystem == 'bottom_right')
    return 'Bottom Right';
}

/**
 * Attach validation to elements in form
 * @param options - configuration array
 */
$.fn.attachValidation = function (options) {
  const element = $(this); //input element

  validatePattern(element, options); //validate on start

  element.on('keyup', function () { //validation on keyup event
    validatePattern(element, options);
  });
};

/**
 * Validation with pattern
 * @param {jquery} elem - jQuery wrapper $(this)
 * @param {object} options - configuration array
 */
function validatePattern(elem, options) {
  const element = elem;
  let pattern = options.pattern || element.attr('pattern');
  let regex = RegExp(pattern);
  let pattern_msg = options.invalidMsg || element.attr('data-pattern-title');
  let valid = regex.test(element.val());
  showValidationResult(element, valid, pattern_msg, options.disableBtnId);
}

/** Validate element with range
 *
 * @param {jquery} jqueryElement - jQuery wrapper $(this)
 * @param {object} options - configuration array
 */
function validateRange(jqueryElement, options) {
  const elementToValidate = jqueryElement;
  const value = parseInt(elementToValidate.val());
  let validResult = false;
  if (!isNaN(value) && options.minVal <= value && value <= options.maxVal) {
    validResult = true;
  }
  showValidationResult(elementToValidate, validResult, options.invalidMsg, options.disableBtnId);
}

/** Show validation result
 *
 * @param {jquery} element - jQuery wrapper $(this)
 * @param {boolean} valid - regex result if is valid
 * @param {string} invalidMsg - error msg displayed in body
 * @param {string} disableBtnId - id of btn to disable
 */
function showValidationResult(element, valid, invalidMsg, disableBtnId) {
  let validation = valid ? '' : 'is-invalid';
  const invalidDiv = '<div class="invalid-feedback text-right mr-3 order-1">' + invalidMsg + '</div>';

  if (validation === '') {
    element.removeClass('is-invalid');
    element.closest('.input-group').find('.invalid-feedback').remove();
  } else {
    element.addClass(validation);
    element.closest('.input-group').find('.invalid-feedback').remove();
    element.closest('.input-group').append(invalidDiv);
    element.closest('.input-group').find('.invalid-feedback').show();
  }
  const element_button_to_disable = $('#' + disableBtnId);
  const invalid_elements_in_form = element.closest('form').find('.is-invalid').length > 0;
  element_button_to_disable.prop('disabled', invalid_elements_in_form);
}

/** Replace special chars - user input sanitization
 * @param {string} string - string is sanitized
 */
function replaceSpecialChars(string) {
  const map = {
    '&': '&amp;',
    '<': '&lt;',
    '>': '&gt;',
    "~": '&#126;',
    '"': '&quot;',
    "'": '&#x27;',
    '`': '&#96;',
    '\\': '&#92;',
    "/": '&#x2F;',
    "(": '&#40;',
    ")": '&#41;',
    ";": '&#59;',
    ".": '&#46;',
    "$": '&#36;',
    "%": '&#37;',
    "#": '&#35;',
    "!": '&#33;',
    "?": '&#63;',
    "|": '&#124;',
    "{": '&#123;',
    "}": '&#125;',
    "+": '&#43;',
  };
  return string.replace(/[&<>~"'`\\/();.$%#!?|{}+]/ig, (match) => (map[match]));
}

function initServiceId() {
  $('#serviceID').text("Service ID: " + serviceArrTmp[activeItem].id);
}

/**
 * Create font list options and set value
 * @param {string} jquerySelector
 * @param {string} value
 * @returns {Promise<void>}
 */
async function createFontList(jquerySelector, value) {
  await createFontListOptions(jquerySelector);
  let existingOption = $(jquerySelector + ' option[value="' + value + '"]');
  if (value === "" || existingOption.length === 0) {
    value = "classic";
  }
  $(jquerySelector + ' option[value="' + value + '"]').attr('selected', 'selected');
  $(jquerySelector).change();
}

/**
 * Create HTML options for select
 * @param {string} jquerySelector
 * @returns {Promise<void>}
 */
async function createFontListOptions(jquerySelector) {
  let fontList = await fontManager.getFileList();
  $(jquerySelector)
    .empty()
    .append(new Option('Classic text', 'classic', false, false))
    .append(new Option('Digital text', 'digital', false, false));

  fontList.forEach(item => {
    $(jquerySelector).append(new Option(item.name, item.path.replace('file://', ''), false, false));
  });
}

/**
 * Create image list options and set value
 * @param {string} jquerySelector
 * @param {boolean} allowUrl
 * @param {string} value
 * @returns {Promise<void>}
 */
async function createImageList(jquerySelector, allowUrl, value) {
  await createImageListInOption(jquerySelector, allowUrl);
  if (value === '') { // Not choose image
    $(jquerySelector + ' option:first').attr('selected', 'selected');
  } else {
    $(jquerySelector + ' option[value="' + value + '"]').attr('selected', 'selected');
  }
  $(jquerySelector).change();
}

/**
 * Create HTML options for select
 * @param {string} jquerySelector
 * @param {boolean} allowUrl
 * @returns {Promise<void>}
 */
async function createImageListInOption(jquerySelector, allowUrl) {
  let imageList = await imageManager.getFileList();
  const allowGif = true;
  $(jquerySelector).empty();

  imageList.forEach(item => {
    let path = item.path;
    let isUrl = (item.storage === "url");
    let isGif = path.includes('.gif');
    if (!allowUrl) {
      //in customGraphics, controlRoom is not valid whole url, only without file://
      path = path.replace('file://', '');
    }
    if ((allowUrl || !isUrl) && (allowGif || !isGif)) {
      $(jquerySelector).append(new Option(item.name, path, false, false));
    }
  });
}

function findImageByPath(imageList, imagePath) {
  for (var i = 0; i < imageList.length; i++) {
    if (imageList[i].path === imagePath) {
      return imageList[i];
    }
  }
  return null;
}

function parseBool(str) {
  if (str.length === null) {
    return str === 1;
  } else {
    return str === "true";
  }
}

function hexToRgb(hex) {
  var c;
  if (/^#([A-Fa-f0-9]{3}){1,2}$/.test(hex)) {
    c = hex.substring(1).split('');
    if (c.length == 3) {
      c = [c[0], c[0], c[1], c[1], c[2], c[2]];
    }
    c = '0x' + c.join('');
    return [(c >> 16) & 255, (c >> 8) & 255, c & 255].join(',') + ',1';
  } else {
    throw new Error('bad HEX input');
  }
}

function rgbToHex(rgb) {
  rgb = rgb.match(/^[\s+]?[\s+]?(\d+)[\s+]?,[\s+]?(\d+)[\s+]?,[\s+]?(\d+)[\s+]?/i);
  if (rgb && rgb.length === 4) {
    var hex = "#" + ("0" + parseInt(rgb[1], 10).toString(16)).slice(-2) +
      ("0" + parseInt(rgb[2], 10).toString(16)).slice(-2) +
      ("0" + parseInt(rgb[3], 10).toString(16)).slice(-2);
    return hex;
  } else {
    throw new Error('bad RGBA input');
  }
}

function escapeHtml(html) {
  return html.replace(/&/g, "&amp;").replace(/</g, "&lt;").replace(/>/g, "&gt;");
}

function convertFloatToPercentage(value) {
  return Math.round(value * 100);
}

function convertPercentageToFloat(value, defaultValue) {
  if (value === "") {
    value = defaultValue;
  }
  return parseFloat(value) / 100;
}

function convertDatetimeToTimestamp(datetime) {
  let date = new Date(datetime);
  let output_date = Date.parse(date.toUTCString());
  output_date = output_date / 1000;
  return output_date;
}

function convertTimestampToDatetime(timestamp) {
  let date = new Date(timestamp * 1000);
  const dateArr = [
    date.getFullYear(),
    ("0" + (date.getMonth() + 1)).slice(-2),
    ("0" + date.getDate()).slice(-2)
  ];
  const timeArr = [
    ("0" + date.getHours()).slice(-2),
    ("0" + date.getMinutes()).slice(-2),
    ("0" + date.getSeconds()).slice(-2)
  ];
  const output_string = dateArr.join("-") + " " + timeArr.join(":");
  return output_string;
}

function convertTimeToSeconds(time) {
  time = time.split(":");
  time = Number(time[0]) * 3600 + Number(time[1]) * 60 + Number(time[2]);
  return time;
}

function convertSecondsToTime(input) {
  const hours = Math.floor(input / 3600).toString().padStart(2, '0');
  const minutes = Math.floor(input % 3600 / 60).toString().padStart(2, '0');
  const seconds = Math.floor(input % 60).toString().padStart(2, '0');
  return hours + ":" + minutes + ":" + seconds;
}