const addGlobalEntryToCode = () => {
    const rootId = "GlobalEntry";

    const rootElem = document.createElement("div");
    rootElem.id = rootId;
    rootElem.style.display = "none";

    document.body.appendChild(rootElem);
    window.initializeGlobalReactInstance(rootId);
};

function createReactInitState() {
    let callbacks = [];
    let initialised = false;

    return {
        withInitializedReact: (callback) => {
            if (initialised) return callback();

            callbacks.push(callback);
        },
        markAsInitialized: () => {
            addGlobalEntryToCode();

            callbacks.forEach((callback) => callback());
            callbacks = [];
            initialised = true;
        },
    };
}

window.reactInitState = createReactInitState();

window.initReactComponent = (
    elementId,
    componentId,
    parameters = {},
    onlyOneComponent = false
) => {
    const remove = {
        current: () => {},
    };

    if (!["GlobalEntry"].includes(componentId)) {
        $(window).one("ajax_page_unloaded", () => remove.current());
    }

    window
        .renderWidget(
            document.getElementById(elementId),
            componentId,
            parameters,
            onlyOneComponent
        )
        .then((handler) => (remove.current = () => handler.remove()));
};

window.renderWidget = (...args) =>
    getServiceProviderPromise().then((serviceProvider) =>
        serviceProvider.widgetRenderer.renderWidget(...args)
    );

window.destructReactComponent = (elementId) => {
    getServiceProviderPromise().then((serviceProvider) =>
        serviceProvider.widgetRenderer.removeWidgetsInElement(
            document.getElementById(elementId)
        )
    );
};

// React takes a while to download and initialize.
// We need to wait for that.
let stuffToDoWhenServiceProviderAvailable = [];
function getServiceProviderPromise(fn) {
    if (window.serviceProvider) return Promise.resolve(window.serviceProvider);

    return new Promise((resolve) =>
        stuffToDoWhenServiceProviderAvailable.push(() =>
            resolve(window.serviceProvider)
        )
    );
}

window._onServiceProviderInitialized = () => {
    stuffToDoWhenServiceProviderAvailable.forEach((fn) => fn());
    stuffToDoWhenServiceProviderAvailable = [];
};

// $.ready will wait for react
$.holdReady(true);
getServiceProviderPromise().then(() => $.holdReady(false));
