var app_name = "CamOverlay";
const appNameLowered = app_name.toLowerCase();

var serviceArr = [];
var serviceArrTmp = [];

var activeItem = 0;
var itemForRemove = 0;

var alertID = 0;
var cameraPreviewUrlPath = "/axis-cgi/mjpg/video.cgi";

var cameraResolutions = [];
var resolution = {};

var scheduleTmpArr = [];

var installPage = "installApp.html";
var defaultPage = "settings.html";

var licenseActivated = -1;
var licenseExpire = -1;
var licenseCode = -1;
var licenseType = "";
const licenseBadge = {
    none: "Not available",
    trial: "Free trial",
    full: "Unlimited",
    limited: "Time limited",
    cloud: "Cloud",
};

var previewRequests = [];

var is_ie = /MSIE|Trident/.test(window.navigator.userAgent);

let jsLogger = JsLogger();
window.onerror = jsLogger.onError;
window.onunhandledrejection = jsLogger.onUnhandledRejection;

var utils = new Utils();
var requestManager = new RequestManager();
var serverApi = new ServerApi(requestManager);
var camOverlayApi = new CamOverlayApi(requestManager);
var vapixApi = new VapixApi(requestManager);
var cameraParams = CameraParamsFactory(CameraParams, vapixApi);

let imageManager = ImageManager(appNameLowered);
let fontManager = FontManager(appNameLowered);
let ptzManager = PtzManager();
let controlRoom = ControlRoom();
let configurationTmp = Configuration();
let sharingInterface = SharingInterface();
let diagnostics = Diagnostics();

$(document).ready(async function () {
    $("#addServiceModal .col-md-5 img")
        .mouseover(function () {
            $hover = $(this).next().attr("src");
            $(this).attr("src", $hover);
        })
        .mouseout(function () {
            $original = $(this).attr("original");
            $(this).attr("src", $original);
        });

    await cameraParams.init();

    // Create top application menu
    renderMenu("", cameraParams.getServerUrl());
    $(".addServiceBtn").mouseenter(function () {
        var secondImgSrc = $(this).find(".secondImg").attr("src");
        $(this).find(".firstImg").attr("src", secondImgSrc);
    });
    $(".addServiceBtn").mouseleave(function () {
        var firstImgSrc = $(this).find(".firstImg").attr("original");
        $(this).find(".firstImg").attr("src", firstImgSrc);
    });

    // Replace server links
    replaceServerUrl(cameraParams.getServerUrl());

    // Show CamOverlay version
    $("#versionNumber").html(cameraParams.getAppVersion());

    // Start acap application
    await startAcapApp();

    // Generate service list component
    try {
        // sportzcast was removec from services, remove it if
        // active in previous versions of CO
        serviceArr = (await camOverlayApi.getServices()).filter(
            (service) => service.name !== "sportzcast"
        );
        serviceArrTmp = JSON.parse(JSON.stringify(serviceArr)); // copy array
    } catch (e) {
        showMessage("warning", "Parse settings failed.");
        serviceArr = [];
        serviceArrTmp = [];
    }

    // fix for missing new cameraList key in service arrays
    // copy previous camera value from previus versions of CO
    // remove from config
    serviceArrTmp.forEach((service, index) => {
        if (!service.cameraList) {
            service.cameraList = [0];

            if (service.camera) {
                service.cameraList = [service.camera];
                delete service.camera;
                delete serviceArr[index].camera;
            }
        } else if (service.camera) {
            //double check, get rid of camera param
            delete service.camera;
            delete serviceArr[index].camera;
        }
    });

    createServiceList(serviceArr);
    setServiceItemActive(activeItem);

    if (serviceArrTmp[activeItem] !== undefined) {
        // If empty config array
        let selectedCamera =
            serviceArrTmp[activeItem].cameraList.length === 0 ||
                serviceArrTmp[activeItem].cameraList[0] === -1
                ? 0
                : serviceArrTmp[activeItem].cameraList[0];

        try {
            // Available resolutions for all video channels
            cameraResolutions = createNewResolutionList(
                cameraParams.getCameraResolutionsForCamera(selectedCamera) ||
                (await vapixApi.getParameter(
                    cameraParams.dynamicParamKeys["cameraResolutions"](
                        selectedCamera
                    )
                )) ||
                cameraParams.getCameraResolutions()
            );

            // Default resolution for all video channels
            resolution = createResolutionObject(
                cameraParams.getCameraResolution(selectedCamera) ||
                (await vapixApi.getParameter(
                    cameraParams.dynamicParamKeys["cameraResolution"](
                        selectedCamera
                    )
                ))
            );
        } catch (error) {
            throw new Error(error.message);
        }
    }

    // Set secondary DNS server
    var secondaryDNS = cameraParams.getSecondaryDNS();
    if (secondaryDNS.indexOf("0.0.0.0") != -1) {
        try {
            const parameters = {
                "Network.DNSServer2": "8.8.8.8",
            };
            vapixApi.setParameters(parameters);
            cameraParams.update(parameters);
        } catch (error) {
            throw new Error(error.message);
        }
    }

    // CamOverlay is suported from 9.50
    var fwSupported = [9, 50];
    var fw = cameraParams.getCameraFirmware().split(".");
    for (var i = 0; i < fwSupported.length && i < fw.length; i++) {
        if (parseInt(fw[i]) < fwSupported[i]) {
            showMessage(
                "danger",
                "CamOverlay is compatible with camera firmware " +
                fwSupported.join(".") +
                " or later. Your camera is using " +
                cameraParams.getCameraFirmware() +
                " now. Please check you have the latest firmware on your camera.",
                true
            );
            break;
        } else if (parseInt(fw[i]) > fwSupported[i]) {
            break;
        }
    }

    // Watermark
    let watermark_text = await detectServer(cameraParams.getServerUrl());
    makeWatermarkForBody(watermark_text);

    // Check license
    validateLicense(
        cameraParams.getSerialNumber(),
        cameraParams.getCameraModel(),
        cameraParams.getCameraFirmware(),
        cameraParams.getAppVersion()
    ).then(function (response) {
        if (typeof response === "undefined") {
            licenseVerificationFailed();
        } else {
            if (response.code == 200) {
                var resData = response.data;
                // Save license to camera
                try {
                    const parameters = {
                        "Camoverlay.Signature": resData.Signature,
                        "Camoverlay.Expire": resData.Expire,
                    };
                    vapixApi.setParameters(parameters);
                    cameraParams.update(parameters);
                } catch (error) {
                    throw new Error(error.message);
                }

                licenseActivated = resData.Activated;
                licenseExpire = resData.Expire;
                licenseCode = resData.LicenseCode;
                licenseType = resData.LicenseType;

                // Check camera time
                setTimeout(async () => {
                    try {
                        const data = await camOverlayApi.getCameraTime();
                        if (!data.state) {
                            let message = '';
                            let shouldDisplayDateTimeButton = false;
                            switch (data.reason) {
                                case 'INVALID_TIME':
                                    message = 'The camera has invalid time.';
                                    shouldDisplayDateTimeButton = true;
                                    break;
                                case 'COULDNT_RESOLVE_HOST':
                                    message = 'The camera can’t connect to the Internet. Please check the Internet connection and DNS of the camera. More info is available in the server report.';
                                    break;
                                case 'CONNECTION_ERROR':
                                    message = 'The camera can’t connect to the Internet. Please check the firewall (outgoing TCP port 443). More info is available in the server report.';
                                default:
                                    message = 'Unknown error';
                                    break;
                            }

                            const dateTimeButton = '<a href="#" class="btn btn-light" onclick="window.open(\'/#settings/system/localization/dateTime\', \'_blank\')">Date & Time</a>';


                            showMessage(
                                "warning",
                                `${message} ${shouldDisplayDateTimeButton ? dateTimeButton : ''}`,
                                true
                            );
                        }
                    } catch (error) {
                        showMessage(
                            "warning",
                            'Could not validate camera time. <a href="#" class="btn btn-light" onclick="window.open(\'/#settings/system/localization/dateTime\', \'_blank\')">Date & Time</a>',
                            true
                        );
                    }
                }, 10000);

                // Check trial
                const expire = parseInt(resData.Expire);
                const isTrialOrLimited =
                    licenseType === "trial" || licenseType === "limited";
                if (isTrialOrLimited && expire != 0) {
                    var msg =
                        'Camera has a <strong style="text-transform:lowercase">' +
                        licenseBadge[licenseType] +
                        "</strong> license which expires on " +
                        getDate(expire) +
                        ".";
                    msg +=
                        ' <button class="btn btn-light" onclick="showLicenseModal(true, true)">Upgrade</button>';
                    showMessage("warning", msg, true);
                }

                // Check latest version
                var lastVersion = resData.LatestACAPVersion.split(".");

                if (
                    lastVersion.length === 0 ||
                    (lastVersion.length === 1 && parseInt(lastVersion) === 0)
                ) {
                    console.warn("Package was not assigned to camera model.");
                }

                var currentVersionParam = cameraParams.getAppVersion();
                var currentVersion = currentVersionParam.split(".");

                for (
                    var i = 0;
                    i < lastVersion.length && i < currentVersion.length;
                    i++
                ) {
                    if (
                        parseInt(currentVersion[i]) < parseInt(lastVersion[i])
                    ) {
                        showMessage(
                            "info",
                            "New " +
                            app_name +
                            " App " +
                            resData.LatestACAPVersion +
                            ' is available. <a href="#" class="btn btn-light" onclick="prepareUpdate(\'' +
                            cameraParams.getServerUrl() +
                            "', '" +
                            resData.LatestACAPVersion +
                            "', '" +
                            currentVersionParam +
                            "')\">Update now</a>",
                            true
                        );
                        break;
                    } else if (
                        parseInt(currentVersion[i]) > parseInt(lastVersion[i])
                    )
                        break;
                }

                checkAcapProductionVersion(cameraParams.getAppVersionHash());

                // Show add service window
                if (serviceArr.length == 0 && !is_ie) {
                    $("#addServiceModal").modal({ show: true });
                }
            } else if (response.code == 489) {
                // Camera is not licensed
                showLicenseModal(false, true);
                showExpiredLicense();
                try {
                    const parameters = {
                        "Camoverlay.Signature": "",
                    };
                    vapixApi.setParameters(parameters);
                    cameraParams.update(parameters);
                } catch (error) {
                    throw new Error(error.message);
                }
            } else {
                licenseVerificationFailed();
            }
        }

        // When CO is turned on/off via API, UI must reflect and change global param 'enabled'
        if (cameraParams.getAppEnabled() === 0) {
            turnOffAllServices();
        }

        const [shouldOpenControlRoom, shouldHideClose] =
            controlRoom.shouldOpen();
        if (shouldOpenControlRoom) {
            showControlRoomCallback(shouldHideClose);
        }

        $("#removeServiceModal").on("shown.bs.modal", () => {
            document.removeEventListener(
                "keypress",
                handleKeyPressOnRemoveServiceBtn
            );
            document.addEventListener(
                "keypress",
                handleKeyPressOnRemoveServiceBtn
            );
        });
        $(".addServiceBtn").click(serviceAddClickedCallback);
        $("#removeServiceBtn").click(serviceRemoveClickedCallback);
        $("#saveBtn").click(saveBtnCallback);
        $("#discardOkBtn").click(discardBtnCallback);
        $("#scheduleAddBtn").click(scheduleAddClickedCallback);
        $("#scheduleSaveBtn").click(scheduleSaveClickedCallback);
        $("#licenseInfoBtn").click(showLicenseInfoCallback);
        $("#imageManagerBtn").click(showImageManagerCallback);
        $("#downloadReport").click(diagnosticsDownloadReport);
        $("#fontManagerBtn").click(showFontManagerCallback);
        $("#controlRoomBtn").click(() => showControlRoomCallback());
    });
});

let handleKeyPressOnRemoveServiceBtn = (event) => {
    if (event.key === "Enter") {
        $("#removeServiceBtn").trigger("click");
    }
};

/**
 * Check if version of package is production
 * @param {string} version_hash
 */
let checkAcapProductionVersion = (version_hash) => {
    let branch_of_acap = version_hash.split("|")[0];
    let is_production_version = branch_of_acap.indexOf("release") > -1;
    if (!is_production_version) {
        showMessage(
            "warning",
            "Note that you are using an unofficial version of the app from branch: <strong>" +
            branch_of_acap +
            "</strong>",
            true
        );
    }
};

const ReportMsg = () => {
    const reportModal = $("#reportModal");

    let show = (msg) => {
        updateText(msg);
        reportModal.modal("show");
    };

    let updateText = (msg) => {
        reportModal.find(".text-msg").text(msg);
    };

    let hide = () => {
        reportModal.find(".text-msg").text("");
        reportModal.modal("hide");
    };

    return Object.freeze({
        show,
        updateText,
        hide,
    });
};

let diagnosticsDownloadReport = async () => {
    const reportMsg = ReportMsg();
    const configArr = JSON.stringify(serviceArr);
    const configArrTmp = JSON.stringify(serviceArrTmp);

    try {
        reportMsg.show("Preparing report for download");
        await diagnostics.downloadReport(reportMsg, configArr, configArrTmp);
    } catch (err) {
        reportMsg.hide();
        showMessage(
            "error",
            "Failed to create diagnostics report",
            false,
            "CreateReportError"
        );
        console.error(err);
    }
    reportMsg.hide();
};

let detectServer = async (serverUrl) => {
    let isTesting = new RegExp("^(https|http)://testing*.*.*$").test(serverUrl);
    let isDev = new RegExp("^(https|http)://*.*.dev$").test(serverUrl);
    if (!isTesting && !isDev) {
        return "";
    }
    let host = new URL(serverUrl).host;
    host = host.split(".");
    if (host.length < 3) {
        return "";
    }
    if (isTesting) {
        return "TESTING | " + host[0];
    }
    if (isDev) {
        return "DEV | " + host[0].replace("cs-", "");
    }
};

let makeWatermarkForBody = (watermark_text) => {
    const svg =
        "data:image/svg+xml;utf8,<svg xmlns='http://www.w3.org/2000/svg' version='1.1' height='150px' width='200px'><text x='0' y='100' transform='rotate(-45 60,60)' fill='gainsboro' font-family='arial' font-size='20'>" +
        watermark_text +
        "</text></svg>";
    const body_element = document.getElementsByTagName("body")[0];
    body_element.style.backgroundImage = 'url("' + svg + '")';
    body_element.style.backgroundAttachment = "fixed";
};

function turnOffAllServices() {
    for (var i = 0; i < serviceArrTmp.length; i++) {
        serviceArrTmp[i].enabled = 0;
    }
    saveBtnCallback(true);
}

async function licenseVerificationFailed() {
    let signature;
    try {
        signature =
            cameraParams.get("Camoverlay.Signature") ||
            (await vapixApi.getParameter("Camoverlay.Signature"));
    } catch (error) {
        throw new Error(error.message);
    }

    if (!signature) {
        showLicenseModal(false, false);
    } else {
        showMessage("warning", "License cannot be verified.");
        // Show add service window
        if (serviceArr.length == 0 && !is_ie) {
            $("#addServiceModal").modal({ show: true });
        }
    }
}

function sortFunc(a, b) {
    return b.width - a.width;
}

function nameHelper(service, alternativeName) {
    let name = alternativeName;
    if (service["customName"]) {
        let alternativeNameWithoutHtmlTags = alternativeName.replace(
            /(<([^>]+)>)/gi,
            " "
        );
        name =
            '<span class="no-tooltip" title="' +
            alternativeNameWithoutHtmlTags +
            '">' +
            escapeHtml(service["customName"]) +
            "</span>";
    }
    return name === undefined ? "Unknown" : name;
}

const serviceName = {
    accuweather:
        'Weather<br /><span class="small" style="font-weight: normal;">(AccuWeather)</span>',
    custom: "Images",
    images: "Images",
    ptz: "PTZ<br />Overlay",
    ptzCompass: "PTZ Compass<br />",
    infoticker: "Info<br />Ticker",
    textAndBackground: "Custom<br>Graphics",
    customGraphics: "Custom<br>Graphics",
    pip: "Picture In<br />Picture",
    scripter:
        'Scripter<br /><span class="small" style="font-weight: normal;">(Scripter)</span>',
    screenSharing: "Screen<br />Sharing",
};

function createServiceCardHTML(serviceItem, index) {
    let item = [];
    let itemStart =
        '<div id="serviceListItem' +
        index +
        '" class="serviceListItem draggableService" data-order="' +
        index +
        '" data-id="' +
        serviceItem.id +
        '">';
    let itemRemoveBtn =
        '<div class="removeService" data-toggle="modal" data-target="#removeServiceModal"><span class="fas fa-times" aria-hidden="true" /></div>';
    let itemName =
        '<div class="serviceName">' +
        nameHelper(serviceItem, serviceName[serviceItem.name]) +
        "</div>";
    let itemSwitchContainer =
        '<div id="' +
        serviceItem.name +
        "Enabled" +
        serviceItem.id +
        '" class="switchContainer"></div>';
    let itemEnd = "</div>";

    item.push(itemStart);
    item.push(itemRemoveBtn);
    item.push(itemName);
    item.push(itemSwitchContainer);
    item.push(itemEnd);

    return item.join("");
}

function createServiceList(serviceArr) {
    let list = [];

    serviceArr.forEach((item, index) => {
        let htmlItemOutput = createServiceCardHTML(item, index);
        list.push(htmlItemOutput);
    });

    let serviceListJqueryObject = $("#serviceList");
    serviceListJqueryObject.html(list.join(""));

    // Add callbacks
    $(".serviceListItem").click(serviceItemClickedCallback);
    // Sortable .serviceListItem
    if ($(window).width() > 769) {
        // Disable on small devices
        Sortable.create(serviceListJqueryObject[0], {
            handle: ".draggableService",
            filter: "#serviceListItemAdd",
            preventOnFilter: true,
            draggable: ".draggableService",
            dataIdAttr: "data-id",
            animation: 150,
            ghostClass: "draggableElementShadow",
            store: {
                set: function (sortable) {
                    saveNewOrderOfServices(sortable.toArray());
                },
            },
        });
    }

    const position = { x: 0, y: 0 };

    interact("#sharingInterfaceContainer")
        .resizable({
            // resize from all edges and corners
            edges: { left: true, right: true, bottom: true, top: true },
            listeners: {
                move(event) {
                    const item = event.target;
                    // update the with and height
                    item.style.width = event.rect.width + "px";
                    item.style.height = event.rect.height + "px";
                },
            },
        })
        .draggable({
            listeners: {
                start(event) {
                    const item = event.target;
                    const posX = item.style.left.replace("px", "");
                    const posY = item.style.top.replace("px", "");
                    // set starting position attributes
                    item.setAttribute("data-x", posX);
                    item.setAttribute("data-y", posY);
                },
                move(event) {
                    const item = event.target;
                    const dataX = item.getAttribute("data-move-x");
                    const dataY = item.getAttribute("data-move-y");
                    const initialX = parseFloat(dataX) || 0;
                    const initialY = parseFloat(dataY) || 0;
                    const deltaX = event.dx;
                    const deltaY = event.dy;
                    const newX = initialX + deltaX;
                    const newY = initialY + deltaY;
                    // set new data-move position
                    item.setAttribute("data-move-x", newX);
                    item.setAttribute("data-move-y", newY);
                    // translate the element
                    item.style.transform =
                        "translate(" + newX + "px, " + newY + "px)";
                },
            },
            modifiers: [
                interact.modifiers.restrictRect({
                    restriction: "body",
                    endOnly: true,
                }),
            ],
        });
    $(".serviceListItemActive").removeClass("serviceListItemActive");
    $("#serviceListItem" + activeItem).addClass("serviceListItemActive");

    serviceArr.forEach((item, index) => {
        const elementId = "#" + item.name + "Enabled" + item.id;
        const switchOnOff = new Switch();
        switchOnOff.destroy(elementId);
        switchOnOff.init(elementId, item.enabled, configurationTmp);
    });
}

function setServiceItemActive(item) {
    activeItem = Number(item);
    $(".serviceListItemActive").removeClass("serviceListItemActive");
    $("#serviceListItem" + item).addClass("serviceListItemActive");

    // Show no service label
    if (!serviceArrTmp.length) {
        $("#serviceContent").html(
            '<div class="noServiceAlert"><span class="h4">No service created.</span></div>'
        );
        return;
    } else if (item > serviceArrTmp.length - 1) {
        setServiceItemActive(serviceArrTmp.length - 1);
        return;
    }

    // Load form for service
    $("#serviceContent").html("");

    // Unregister events from controls of form
    $(".form-control").unbind();
    $(".form-control-inline").unbind();
    // Unregistrer events from unexisting items of DOM (draggable lib)
    if (typeof interact !== "undefined") {
        interact(".boxIn.active").unset();
    }
    // Unregister events from fileManager for specific service
    $("body").off("hide.bs.modal");

    const serviceFilePath = getServicePage(serviceArrTmp[item].name);
    if (serviceFilePath.length > 0) {
        $("#serviceContent").load(serviceFilePath, function () {
            $("[data-toggle='tooltip'], .toggle-tooltip")
                .not(".no-tooltip")
                .tooltip({ delay: { show: 500, hide: 0 } });
        });
    }
}

function getServicePage(name) {
    const convertNameToPath = {
        accuweather: "accuweather/accuweather.html",
        infoticker: "infoticker/infoticker.html",
        custom: "custom/custom.html", // 'custom' service renamed to 'images'
        images: "custom/custom.html",
        ptz: "ptz/ptz.html",
        ptzCompass: "compass/compass.html",
        pip: "pip/pip.html",
        scripter: "scripter/scripter.html",
        textAndBackground: "customGraphics/customGraphics.html", // 'textAndBackground' service renamed to 'customGraphics'
        customGraphics: "customGraphics/customGraphics.html",
        screenSharing: "screenSharing/screenSharing.html",
    };
    const serviceName = convertNameToPath[name];
    const errMsg =
        'Unknown service: <span class="text-uppercase">' + name + "</span>";
    if (serviceName === undefined) {
        showMessage("danger", errMsg, true, "unknownService");
        return;
    }
    return serviceName;
}

async function saveNewOrderOfServices(order) {
    let newServiceArrTmp = [];
    order.forEach(function (item, i) {
        serviceArrTmp.forEach(function (service) {
            if (service.id === parseInt(item)) {
                newServiceArrTmp.push(service);
            }
        });
    });

    serviceArrTmp = JSON.parse(JSON.stringify(newServiceArrTmp));
    serviceArr = JSON.parse(JSON.stringify(newServiceArrTmp));
    reorderServiceCards(newServiceArrTmp);
    await saveBtnCallback(true, newServiceArrTmp);
    const newActiveItem = parseInt(
        $(".serviceListItemActive").attr("data-order")
    );
    activeItem = newActiveItem;

    $(".serviceListItem").off("click").on("click", serviceItemClickedCallback);
}

function reorderServiceCards(serviceList) {
    for (let i = 0; i < serviceList.length; i++) {
        const serviceEnabled = serviceList[i].enabled;
        const serviceId = serviceList[i].id;
        const serviceName = serviceList[i].name;
        const serviceItem = $(
            '#serviceList .serviceListItem[data-id="' + serviceId + '"]'
        );
        if (serviceItem.length > 0) {
            serviceItem.attr("id", "serviceListItem" + i);
            serviceItem.attr("data-order", i);
        }
        const elementId = "#" + serviceName + "Enabled" + serviceId;
        const switchOnOff = new Switch();
        switchOnOff.destroy(elementId);
        switchOnOff.init(elementId, serviceEnabled, configurationTmp);
    }
}

function serviceItemClickedCallback(event) {
    var targ = event.target || event.srcElement;
    var item = $(targ).closest(".serviceListItem");

    var itemID = item[0].id.substring(15);
    if (itemID == "Add") {
        return;
    } else if ($(targ).closest("div").hasClass("removeService")) {
        itemForRemove = itemID;
        return;
    } else if ($(targ).closest("div").hasClass("switchContainer")) {
        return;
    }

    setServiceItemActive(itemID);
}

async function serviceAddClickedCallback(event) {
    const target = event.target || event.srcElement;
    const item_to_add = $(target).closest(".addServiceBtn").prop("name");
    const allowed_services = [
        "accuweather",
        "images",
        "ptzCompass",
        "infoticker",
        "ptz",
        "pip",
        "customGraphics",
        "screenSharing",
    ];

    let can_add = allowed_services.includes(item_to_add);
    if (!can_add) {
        throw Error("Can't add unknown service.");
    }

    await addDefaultService(item_to_add);

    createServiceList(serviceArrTmp);
    setServiceItemActive(serviceArrTmp.length - 1);

    await saveBtnCallback(true, serviceArrTmp);
}

async function addDefaultService(serviceName) {
    const allow = await configurationTmp.canAddNewService();
    if (!allow) {
        const allow_add_service_msg =
            await configurationTmp.getAddNewServiceMessage();
        showMessage("info", allow_add_service_msg, false, "canAddNewService");
    }

    if (serviceName === "infoticker") {
        serviceArrTmp.push({
            id: getUniqueID(),
            enabled: 0,
            automationType: "manual",
            cameraList: [0],
            schedule: "",
            name: "infoticker",
            customName: "",
            showClock: 1,
            clockType: "24h",
            textColor: "0,0,0,1",
            bgColor: "255,255,255,1",
            weatherLocation: "", // location key
            weatherLocationName: "", // title from location api
            weatherLang: "en-us",
            weatherUnits: "Metric",
            numberOfLines: 1,
            switchingTime: 5,
            crawlLeft: false,
            crawlSpeed: 1.0,
            coordSystem: "bottom", // bottom|top values
            pos_y: 0.05, // in percentage
            width: 1920,
            height: 1080,
            zIndex: 0,
            font: "classic",
            fontSize: 40,
            sourceType: "text", // text|url
            source: "",
        });
    } else if (serviceName === "accuweather") {
        serviceArrTmp.push({
            id: getUniqueID(),
            enabled: 0,
            automationType: "manual",
            cameraList: [0],
            schedule: "",
            name: "accuweather",
            customName: "",
            location: "", // location key
            locationName: "", // title from location api
            title: "", // title from user
            bgStartColor: "0,0,0,0.75",
            bgEndColor: "0,0,0,0.75",
            clockType: "24h", // 12h|24h format
            lang: "en-us",
            font: "classic",
            units: "Metric",
            pos_x: 10,
            pos_y: 10,
            width: 1920,
            height: 1080,
            zIndex: 0,
            coordSystem: "top_right",
            layout: "0",
            scale: 1.0,
        });
    } else if (serviceName === "ptzCompass") {
        serviceArrTmp.push({
            id: getUniqueID(),
            enabled: 0,
            automationType: "manual",
            cameraList: [0],
            schedule: "",
            name: "ptzCompass",
            customName: "",
            pos_x: 10,
            pos_y: 10,
            width: 1920,
            height: 1080,
            zIndex: 0,
            coordSystem: "top_left",
            scale: 1.0,
            type: "compass", // compass|image
            image: "", // file:///usr/local/packages/camoverlay/localdata/user_images/vodnik-1.png
            northPan: 0.0, // pan in degrees
            cameraPosX: 0, // used only if type is 'map' or 'image'
            cameraPosY: 0,
            colorScheme: "black", // black|white|orange
        });
    } else if (serviceName === "images") {
        // old name was 'custom'
        serviceArrTmp.push({
            id: getUniqueID(),
            enabled: 0,
            automationType: "time", // manual|time
            cameraList: [0],
            schedule: "",
            name: "images",
            customName: "",
            width: 1920,
            height: 1080,
            zIndex: 0,
            overlayList: [],
        });
    } else if (serviceName === "ptz") {
        serviceArrTmp.push({
            id: getUniqueID(),
            enabled: 0,
            automationType: "manual",
            cameraList: [0],
            schedule: "",
            name: "ptz",
            customName: "",
            width: 1920,
            height: 1080,
            zIndex: 0,
            ptz_positions: {},
        });
    } else if (serviceName === "pip") {
        serviceArrTmp.push({
            id: getUniqueID(),
            enabled: 0,
            automationType: "manual",
            cameraList: [0],
            schedule: "",
            name: "pip",
            customName: "",
            width: 1920,
            height: 1080,
            zIndex: 0,
            coordSystem: "top_left",
            pos_x: 10,
            pos_y: 10,
            fps: 1,
            compression: 50,
            screenSize: 500,
            source_type: "axis_camera", // axis_camera|mjpeg_url|ws
            mjpeg_url: "",
            camera_ip: "",
            camera_port: 80,
            camera_user: "root",
            camera_pass: "",
            camera_width: 320,
            camera_height: 240,
            camera_view_area: "1",
            camera_overlay_params: "overlays=all",
        });
    } else if (serviceName === "customGraphics") {
        // old name was 'textAndBackground'
        serviceArrTmp.push({
            id: getUniqueID(),
            enabled: 0,
            automationType: "manual",
            cameraList: [0],
            schedule: "",
            name: "customGraphics",
            customName: "",
            pos_x: 10,
            pos_y: 10,
            width: 1920,
            height: 1080,
            zIndex: 0,
            coordSystem: "top_left",
            clockFormat: "24h",
            background: "custom", // custom|image
            image: "", // /usr/local/packages/camoverlay/localdata/user_images/vodnik-1.png
            customAreaColor: "0,0,0,0.8",
            customAreaWidth: 300,
            customAreaHeight: 200,
            customAreaCorners: "rounded",
            mappingZones: [
                {
                    name: "field1",
                    pos_x: 10,
                    pos_y: 10,
                    type: "plain",
                    text: [
                        {
                            source: "",
                            active: true,
                        },
                    ],
                    wrapText: false,
                    textLines: 1,
                    textWidth: 200,
                    textAlign: "A_LEFT",
                    textColor: "255,0,0,1.0",
                    font: "classic",
                    fontSize: 30,
                    switchingTime: 0,
                },
            ],
        });
    } else if (serviceName === "screenSharing") {
        serviceArrTmp.push({
            id: getUniqueID(),
            enabled: 0,
            automationType: "manual",
            schedule: "",
            cameraList: [0],
            name: "screenSharing",
            customName: "",
            pos_x: 10,
            pos_y: 10,
            width: 1920,
            height: 1080,
            zIndex: 0,
            coordSystem: "top_left",
            screenSize: 500,
            fps: 1,
        });
    }
}

function getUniqueID() {
    // Get max id from all services
    var maxID = 0;
    for (var i = 0; i < serviceArrTmp.length; i++) {
        if (serviceArrTmp[i].id != undefined && serviceArrTmp[i].id > maxID)
            maxID = serviceArrTmp[i].id;
    }
    return maxID + 1;
}

async function serviceRemoveClickedCallback() {
    // Handle error
    if (serviceArrTmp[itemForRemove] === undefined) {
        $("#removeServiceModal").modal("hide");
        return;
    }

    // Stop running share
    const service_id = serviceArrTmp[itemForRemove].id;
    const running_share = sharingInterface.isRunning(service_id);
    if (running_share) {
        sharingInterface.stop(service_id);
    }

    try {
        serviceArr = await camOverlayApi.getServices();
    } catch (error) {
        throw new Error(error.message);
    }

    serviceArrTmp.splice(itemForRemove, 1);
    serviceArr.splice(itemForRemove, 1);
    let backupActiveItem = activeItem;
    if (backupActiveItem > itemForRemove) {
        backupActiveItem--;
    }

    setServiceItemActive(backupActiveItem);
    createServiceList(serviceArrTmp);
    saveBtnCallback(true, serviceArr);
}

const purgePtzPositionsOnSave = async () => {
    const numServices = serviceArrTmp.length;
    for (let i = 0; i < numServices; i++) {
        if (serviceArrTmp[i].name === "ptz") {
            if (typeof serviceArrTmp[i].cameraList[0] === undefined) return;
            let ptzList;
            try {
                ptzList = await ptzManager.getList(serviceArrTmp[i].cameraList[0]);
            } catch (e) {
                console.error('Could not get ptzlist');
                return;
            }
            let allowedPtzpositions = {};
            for (const [ptzName, valObj] of Object.entries(serviceArrTmp[i].ptz_positions)) {
                if (ptzList.includes(ptzName)) {
                    allowedPtzpositions[ptzName] = valObj;
                }
            }
            serviceArrTmp[i].ptz_positions = allowedPtzpositions;
        }
    }
}

async function saveBtnCallback(silence, services) {
    if (!services) {
        await purgePtzPositionsOnSave()
        serviceArr = JSON.parse(JSON.stringify(serviceArrTmp));
    } else {
        serviceArr = JSON.parse(JSON.stringify(services));
    }

    // Save params to camera
    let servicesObj = { services: serviceArr };
    let servicesStr = JSON.stringify(servicesObj);

    const save_msg = {
        error: "Parameters setting error.",
        success: "Parameters have been successfully updated.",
    };
    let response = null;
    try {
        response = await camOverlayApi.setServices(servicesStr);
    } catch (err) {
        if (silence !== true) {
            showMessage("danger", save_msg.error, false, "saveParams");
        }
        return false;
    }

    if (response.message !== "OK") {
        if (silence !== true) {
            showMessage("danger", save_msg.error, false, "saveParams");
        }
        return false;
    }

    if (silence !== true) {
        hideSaveWarning();
        showMessage("success", save_msg.success, false, "saveParams");
        return true;
    }

    let enabledServices = getEnabledServices();

    // When one service is turned on, set global param 'enabled' to 1
    if (enabledServices.length > 0) {
        try {
            const parameters = {
                "Camoverlay.Enabled": "1",
            };
            vapixApi.setParameters(parameters);
            cameraParams.update(parameters);
        } catch (error) {
            throw new Error(error.message);
        }
    }

    // Send statistic data
    serverApi.sendStatistics(
        cameraParams.getServerUrl(),
        cameraParams.getSerialNumber(),
        {
            services: enabledServices,
        }
    );
}

function discardBtnCallback() {
    serviceArrTmp = JSON.parse(JSON.stringify(serviceArr));
    setServiceItemActive(activeItem);
    createServiceList(serviceArrTmp);
}

/**
 * Camera supports PTZ
 * @returns {Promise<boolean>}
 */

async function supportsPtz() {
    let result;
    try {
        result =
            cameraParams.get("Properties.PTZ.PTZ") ||
            (await vapixApi.getParameter("Properties.PTZ.PTZ"));
    } catch (error) {
        throw new Error(error.message);
    }

    if (!result) return false;
    return true;
}

/**
 * Create url for image preview from camera
 * @param {string} elementId - set att src to this element
 * @param {Object} resolution - contains width and height prop
 * @param {?number} videoChannel - number of channel of camera
 */
async function showImageFromCamera(elementId, resolution, videoChannel) {
    if (videoChannel === null || videoChannel === undefined) {
        return console.warn("Null as videoChannel");
    }

    let cameraParam;
    try {
        cameraParam =
            cameraParams.getCameraSource(videoChannel) ||
            (await vapixApi.getParameter(
                cameraParams.dynamicParamKeys["cameraSource"](videoChannel)
            ));
    } catch (error) {
        throw new Error(error.message);
    }

    let camera =
        cameraParam === "quad" ? "quad" : String(Number(videoChannel) + 1);

    window.initReactComponent(
        elementId,
        "VideoPlayer",
        {
            resolution,
            camera,
        },
        true
    );
}

function initTooltips() {
    let tooltips = getFormTooltips();
    insertTooltip(
        $('select[id*="VideoChannel"]').closest(".input-group"),
        tooltips["videochannel"]
    );
    $("[data-toggle='tooltip'], .toggle-tooltip")
        .not(".no-tooltip")
        .tooltip({ delay: { show: 500, hide: 0 } });
}

function insertTooltip(element, tooltip) {
    element.attr("data-toggle", "tooltip");
    element.attr("title", tooltip);
}

function getFormTooltips() {
    return {
        // artpec 6 and higher
        enabled:
            "Switch ON if you want to show graphics overlay for this service. You can run up to 4 services at the same time in one Video Channel.",
        videochannel:
            "If you want to stream independent graphics for a particular stream in the CamStreamer App, you'll have to create an independent View Area. Before selecting a new service in the CamStreamer App, select a particular Video Channel in the drop-down box at the top of the page.",
    };
}

function getEnabledServices() {
    var serviceNames = "";
    for (var i = 0; i < serviceArr.length; i++) {
        if (serviceArr[i].enabled === 1) {
            if (serviceNames.length) serviceNames += ",";
            serviceNames += serviceArr[i].name;
        }
    }
    return serviceNames;
}

function createScheduleBtn() {
    let btnIcon = '<span class="fa fa-calendar-alt mr-1"></span>';
    let btnDesc = "Set schedule";
    if (
        serviceArrTmp[activeItem].schedule != undefined &&
        serviceArrTmp[activeItem].schedule.length
    ) {
        btnDesc = "Edit schedule";
    }
    $("#scheduleBtn").html(btnIcon + btnDesc);
    $("#scheduleBtn").unbind().click(scheduleClickedCallback);
    $("#fromDay").unbind().click(scheduleFromDayChangeCallback);
    $("#fromDay").change(scheduleFromDayChangeCallback);
}

function scheduleClickedCallback() {
    $("#scheduleModal").modal({ show: true });
    scheduleTmpArr = [];
    if (serviceArrTmp[activeItem].schedule != undefined)
        scheduleTmpArr = serviceArrTmp[activeItem].schedule.split("|");
    createScheduleTable(scheduleTmpArr);
}

function createScheduleTable(scheduleArr) {
    var table = "<tbody>";
    if (scheduleArr.length) {
        for (var i = 0; i < scheduleArr.length; i++) {
            var interval = scheduleArr[i].trim();
            if (interval.length == 0) {
                scheduleTmpArr = [];
                createScheduleTable(scheduleTmpArr);
                return;
            }
            var intervalArr = interval.split("..");
            table += '<tr class="schedule" data-schedule="' + interval + '">';
            table +=
                "<td>" +
                intToDay(parseInt(intervalArr[0][0])) +
                " " +
                intervalArr[0].substring(2) +
                "</td>";
            table += "<td> - </td>";
            table +=
                "<td>" +
                intToDay(parseInt(intervalArr[1][0])) +
                " " +
                intervalArr[1].substring(2) +
                "</td>";
            table +=
                '<td><button id="removeSchedule' +
                i +
                '" class="btn btn-sm btn-primary removeSchedule">Remove</button></td>';
            table += "</tr>";
        }
    } else
        table +=
            '<tr class="schedule"><td colspan="4">No schedule set.</td></tr>';

    table += "</tbody>";

    $("#scheduleList").html(table);
    $(".removeSchedule").click(removeScheduleClickedCallback);
}

function scheduleFromDayChangeCallback() {
    $("#toDay").val($("#fromDay").val());
}

function scheduleAddClickedCallback() {
    var fromDay = $("#fromDay")[0].selectedIndex + 1;
    var fromTime = $("#fromTime").val();
    var toDay = $("#toDay")[0].selectedIndex + 1;
    var toTime = $("#toTime").val();
    scheduleTmpArr.push(
        fromDay +
        " " +
        validateTime(fromTime, "00:00:00") +
        ".." +
        toDay +
        " " +
        validateTime(toTime, "23:59:59")
    );
    createScheduleTable(scheduleTmpArr);
    return false; // Prevent form submission
}

function validateTime(inputTime, defaultVal) {
    var tArr = inputTime.split(":");
    var tArrValidated = [];
    for (var i = 0; i < 3; i++) {
        if (i < tArr.length) {
            var num = parseInt(tArr[i]);
            if (!Number.isNaN(num)) {
                tArrValidated.push(("0" + num).slice(-2));
            } else {
                console.error(
                    "Time conversion error: " +
                    tArr +
                    ", default value is used: " +
                    defaultVal
                );
                tArrValidated = defaultVal.split(":");
                break;
            }
        } else {
            tArrValidated.push("00");
        }
    }
    return tArrValidated.join(":");
}

function removeScheduleClickedCallback(event) {
    var targ = event.target || event.srcElement;
    var itemPos = targ.id.substring(14);
    scheduleTmpArr.splice(itemPos, 1);
    createScheduleTable(scheduleTmpArr);
}

function scheduleSaveClickedCallback() {
    var schedule = "";
    for (var i = 0; i < scheduleTmpArr.length; i++) {
        schedule += scheduleTmpArr[i];
        if (i < scheduleTmpArr.length - 1) schedule += "|";
    }
    serviceArrTmp[activeItem].schedule = schedule;
    showSaveWarning();
    createScheduleBtn();
}

function intToDay(dayNum) {
    switch (dayNum) {
        case 1:
            return "Mon";
        case 2:
            return "Tue";
        case 3:
            return "Wed";
        case 4:
            return "Thu";
        case 5:
            return "Fri";
        case 6:
            return "Sat";
        case 7:
            return "Sun";
        default:
            return "";
    }
}

function showTrialFormClicked() {
    $("#panelLicenseCode")[0].className = "card card-default";
    $("#panelTrial")[0].className = "card card-success";
    $("#licenseContent").load("activateTrialForm.html", function () {
        $("#licenseFormActivateTrial").click(activateTrialClicked);
    });
}

function showLicenseFormClicked() {
    $("#panelLicenseCode")[0].className = "card card-success";
    $("#panelTrial")[0].className = "card card-default";
    $("#licenseContent").load("activateForm.html", function () {
        $("#licenseFormActivate").click(activateClicked);
    });
}

async function activateTrialClicked() {
    var email = encodeURIComponent($("#email").val());
    if (email.length == 0) {
        $("#emailGroup").addClass("has-error");
        return false;
    }

    var dataJson = {
        a: "registerlicense",
        p: "camoverlay",
        serial_number: cameraParams.getSerialNumber(),
        license_code: "TRIAL",
        email: email,
    };

    let response;
    try {
        response = await serverApi.handleLicenses(
            `${cameraParams.getServerUrl()}/index.php`,
            dataJson
        );
    } catch (error) {
        showLicenseModalMessage(error.message);
    }

    if (response.code === 200) {
        showLicenseModalMessage("");
        window.location.reload(); // Reload page
    } else {
        showLicenseModalMessage(response.message);
    }

    return false;
}

async function activateClicked() {
    var license = encodeURIComponent($("#licenseCode").val());
    if (license.length == 0) {
        $("#licenseCodeGroup").addClass("has-error");
        return false;
    }

    license = license.replace(/^\s+|\s+$/gm, "");
    license = license.replace(/%20/g, "");

    var dataJson = {
        a: "registerlicense",
        p: "camoverlay",
        serial_number: cameraParams.getSerialNumber(),
        license_code: license,
    };

    let response;
    try {
        response = await serverApi.handleLicenses(
            `${cameraParams.getServerUrl()}/index.php`,
            dataJson
        );
    } catch (error) {
        showLicenseModalMessage(error.message);
    }

    if (response.code == 200) {
        showLicenseModalMessage("");
        window.location.reload(); // Reload page
    } else {
        showLicenseModalMessage(response.message);
    }

    return false;
}

function getDate(timestamp) {
    const d = new Date(timestamp * 1000);
    return d.getDate() + "." + (d.getMonth() + 1) + "." + d.getFullYear();
}

function showLicenseInfoCallback() {
    let licenseInfo = "No license info.";

    if (licenseActivated != -1 && licenseExpire != -1 && licenseCode != -1) {
        licenseInfo = '<dl class="dl-horizontal">';
        licenseInfo += "<dt>License code</dt><dd>" + licenseCode + "</dd>";
        licenseInfo += "<dt>License type</dt><dd>" + licenseBadge[licenseType];
        let isTrialOrLimited =
            licenseType === "trial" || licenseType === "limited";
        if (isTrialOrLimited && licenseExpire != -1) {
            licenseInfo +=
                ' <button class="ml-2 btn btn-sm btn-primary" onclick="showLicenseModal(true, true);">Upgrade</button>';
        }
        licenseInfo += "</dd>";
        if (isTrialOrLimited && licenseExpire != -1) {
            licenseInfo +=
                "<dt>Expiration date</dt><dd>" +
                getDate(licenseExpire) +
                "</dd>";
        }
        licenseInfo +=
            "<dt>Activated</dt><dd>" + getDate(licenseActivated) + "</dd>";
        licenseInfo += "</dl>";
    }

    $("#licenseInfoBody").html(licenseInfo);
    $("#licenseInfoModal").modal("show");
}

function showExpiredLicense() {
    $(".content").hide();
    $("#imageManagerBtn").hide();
    $(".expired_license").show();
}

function showLicenseModal(disableTrial, allowExit) {
    if (allowExit) {
        $("#licenseModal").attr("data-backdrop", true);
        $("#licenseModalCloseBtn").show();
    } else {
        $("#licenseModal").attr("data-backdrop", "static");
        $("#licenseModalCloseBtn").hide();
    }
    $(".modal").modal("hide");
    $("#licenseModal").modal("show");
    $("#showLicenseForm").unbind().click(showLicenseFormClicked);

    if (disableTrial) {
        $("#showTrialForm").unbind();
        $("#licensePanelTrial").addClass("disabled");
    } else {
        $("#showTrialForm").unbind().click(showTrialFormClicked);
        $("#licensePanelTrial").removeClass("disabled");
    }
}

function showLicenseModalMessage(msg) {
    if (msg.length == 0) {
        $("#licenseModalMessages").html("");
        return;
    }

    var message = '<div class="alert alert-dismissible alert-danger">';
    message +=
        '<button type="button" class="close" data-dismiss="alert">&times;</button>';
    message += msg + "</div>";
    $("#licenseModalMessages").html(message);
}

function showControlRoomCallback(hideCloseBtn = false) {
    const imagePreviewItem = "imagePreviewCamera";
    if ($(`#${imagePreviewItem}`).length > 0) {
        window.destructReactComponent(imagePreviewItem);
    }

    $("body").removeClass("regular_background").addClass("empty_background");
    $(".main_content").hide();

    const controlRoomContainer = $("#controlRoomContainer");
    controlRoomContainer.load("controlRoom.html", function () {
        if (hideCloseBtn) {
            $("#controlRoomClose").hide();
        }
        controlRoomContainer.show();
        controlRoom.init();
        controlRoom.addHash(hideCloseBtn);
    });
}

async function hideControlRoomCallback() {
    const imagePreviewItem = "imagePreviewCamera2";
    if ($(`#${imagePreviewItem}`).length > 0) {
        window.destructReactComponent(imagePreviewItem);
    }

    $("body").removeClass("empty_background").addClass("regular_background");
    // Watermark
    let watermark_text = await detectServer(cameraParams.getServerUrl());
    makeWatermarkForBody(watermark_text);
    $(".main_content").show();
    $("#controlRoomContainer").hide().html("");
    setServiceItemActive(activeItem); // reload whole service, because of changed inputs
    createServiceList(serviceArrTmp);

    controlRoom.removeHash();
}

async function showImageManagerCallback() {
    await imageManager.init();
    imageManager.open();
}

async function showFontManagerCallback() {
    await fontManager.init();
    fontManager.open();
}

async function startAcapApp() {
    let response = await vapixApi.startAcap();

    response = response.trim();

    var errorCode = "";
    var found = response.search("Error:");
    if (found != -1) errorCode = response.substring(7);

    if (response === "OK") {
        setTimeout(Promise.resolve, 2000); // Wait for acap start
    } else if (errorCode == 6) {
        Promise.resolve();
    } else {
        showMessage(
            "danger",
            "Could not start camoverlay package, error: " + errorCode + "."
        );
        Promise.resolve();
    }
}

function showMessage(type, html, static, msgName) {
    var header = "Info:";
    if (type == "success") header = "Success:";
    else if (type == "warning") header = "Warning!";
    else if (type == "danger") header = "Error!";

    var id = alertID++;
    var exists = 0;
    var targetDiv =
        static === false || static === undefined
            ? "#messages_fluid"
            : "#messages";

    if ($(targetDiv).find("." + msgName).length > 0) exists = 1;
    if (exists === 0 || msgName === undefined) {
        if (msgName === undefined) msgName = "";
        var message =
            '<div id="alert' +
            id +
            '" class="alert alert-dismissible alert-' +
            type +
            " " +
            msgName +
            '">';
        if (static !== false) {
            message +=
                '<button type="button" class="close" data-dismiss="alert">&times;</button>';
        }
        message += '<strong class="mr-2">' + header + "</strong>";
        message += html + "</div>";
        $(targetDiv).append(message);
    }

    if (static === undefined || static === false) {
        $("#alert" + id)
            .fadeTo(5000, 500)
            .slideUp(500, function () {
                $("#alert" + id).alert("close");
            });
    }
}

function validateLicense(
    serialNumber,
    cameraModel,
    cameraFirmware,
    appVersion
) {
    var dataJson = {
        a: "checklicense",
        p: "camoverlay",
        serial_number: serialNumber,
        model: cameraModel,
        firmware: cameraFirmware,
        app_ver: appVersion,
    };

    return serverApi.handleLicenses(
        `${cameraParams.getServerUrl()}/index.php`,
        dataJson
    );
}
