var propertiesResponse = null;
var SUPPORTED_VERSION_FOR_REMOTE_PIP_OVERLAYS = '10.5.2';
var lastRequestId = 1001;

$(document).ready(async function () {

  initTooltips();
  initServiceId();

  // Fill form
  initLayerZindex(serviceArrTmp[activeItem].zIndex || 0);

  initDisplayedOverlaysSelect();

  $('#pipCustomName')
    .val(serviceArrTmp[activeItem].customName || '')
    .keyup(onCustomNameKeyUp)
    .attachValidation({
      pattern: '^[A-Za-z0-9 -]{0,}$',
      invalidMsg: 'Only chars A-Z, a-z, 0-9, white space and dash are allowed.',
      disableBtnId: 'saveBtn'
    });

  $('#pipPosX').val(serviceArrTmp[activeItem].pos_x);
  $('#pipPosY').val(serviceArrTmp[activeItem].pos_y);

  $('#pipFPS').val(serviceArrTmp[activeItem].fps || 1);
  $('#pipCompression').val(serviceArrTmp[activeItem].compression || '50');
  $('#pipMaxSize').val(serviceArrTmp[activeItem].screenSize || 500);

  initPipSourceType();

  const resolutionId = '#pipResolution';
  const videoChannelId = '#pipVideoChannel';
  const videoPreviewSelectId = '#pipPreviewVideoChannel';
  createResolutionList(resolutionId);
  onResolutionChange(resolutionId, videoChannelId);
  createPositionList('#pipPosition', serviceArrTmp[activeItem].coordSystem);
  await createTriggeringElement('#pipAutomationTypeContainer', serviceArrTmp[activeItem].automationType, serviceArrTmp[activeItem].schedule, false);
  onVideoChannelChange(resolutionId, videoChannelId, videoPreviewSelectId);
  await createVideoChannelElement(videoChannelId, serviceArrTmp[activeItem].cameraList);
  onPreviewVideoChannelChange(videoPreviewSelectId, resolutionId);

  let currentCameraResolutionObject = cameraResolutions[parseInt($('#pipResolution').val())];
  showImageFromCamera('imagePreviewCamera', currentCameraResolutionObject, $(videoPreviewSelectId).val());

  $('#pipCameraIP').change((e) => {
    e.stopPropagation();
    $('#checkCameraConnection').click();
  });

  // Callbacks for changes
  $('.form-control').change(formChangedCallback);

  $('#webcamSharingStart').attr('data-service-id', serviceArrTmp[activeItem].id).click(sharingStart);
  $('#webcamSharingStop').attr('data-service-id', serviceArrTmp[activeItem].id).click(sharingStop);
  $('#webcamSharingChange').attr('data-service-id', serviceArrTmp[activeItem].id).click(sharingChange);
  $('#webcamSharingShowPreview').attr('data-service-id', serviceArrTmp[activeItem].id).click(sharingShowPreview);
  $('#stateCapture').attr('data-service-id', serviceArrTmp[activeItem].id);

  $('#pipMaxSize')
    .change(handleChangeMaxSize)
    .attachValidation({
      pattern: '^[1-9][0-9]{0,}$',
      invalidMsg: 'The value must not be empty or zero.',
      disableBtnId: 'webcamSharingStart'
    });

  $('#pipFPS')
    .change(handleChangeFps)
    .attachValidation({
      pattern: '^([1-9]|[12][0-9]|30)$',
      invalidMsg: 'The value must be 1 to 30.',
      disableBtnId: 'webcamSharingStart'
    });

  // Handle screen sharing state
  const isRunning = sharingInterface.isRunning(serviceArrTmp[activeItem].id);
  isRunning ? sharingOn() : sharingOff();

  const isVisiblePreview = await sharingInterface.isPreviewVisible(serviceArrTmp[activeItem].id);
  if (isRunning && !isVisiblePreview) {
    $('#webcamSharingShowPreview[data-service-id="' + serviceArrTmp[activeItem].id + '"]').show();
  }
  const isAllowedChangeCamera = await sharingInterface.isAllowedChangeCamera(serviceArrTmp[activeItem].id);
  if (isRunning && isAllowedChangeCamera) {
    sharingChangeOption();
  }

  // prevent page from refreshing on submitting data
  document.getElementById('cameraProxyInfoForm').addEventListener('submit', (e) => {
    e.preventDefault();
  });
  document.querySelectorAll('#cameraProxyInfoForm input[required]').forEach((input) => {
    input.addEventListener('invalid', (e) => {
      e.target.setCustomValidity('This field is required!')
    });
    input.addEventListener('input', (e) => {
      e.target.setCustomValidity('')
    });
  });


  $('#checkCameraConnection').on('click', function (e) {
    // check if form is valid (all required fields are filled)
    if (!e.target.closest('form').checkValidity()) return;

    getProxyCameraData(100)
      .then(function () {

        showConnectionState(true, "Connection successful");
      }).catch(function (err) {

        showConnectionState(false, "Cannot connect to camera. Check the details of the remote camera.");
        console.error("Cannot connect to camera. Check the details of the remote camera.", err);
      });
  });

  if (serviceArrTmp[activeItem].source_type === "axis_camera") {
    if (serviceArrTmp[activeItem].camera_ip.length && serviceArrTmp[activeItem].camera_user.length && serviceArrTmp[activeItem].camera_pass.length) {
      getProxyCameraData(4000)
        .catch(function (err) {

          showConnectionState(false, "Cannot connect to camera. Check the details of the remote camera.");
          console.error("Cannot connect to camera. Check the details of the remote camera.", err);
        });
    } else {
      $("#cameraInfoForm").hide();
    }
  }

  formChangedCallback();

  createScheduleBtn();
});

function initPipSourceType() {
  // Fill data
  $('#pipMjpegUrl').val(serviceArrTmp[activeItem].mjpeg_url);
  $('#pipCameraWidth').val(serviceArrTmp[activeItem].camera_width);
  $('#pipCameraHeight').val(serviceArrTmp[activeItem].camera_height);
  $('#pipCameraIP').val(serviceArrTmp[activeItem].camera_ip);
  $('#pipCameraPort').val(serviceArrTmp[activeItem].camera_port);
  $('#pipCameraUser').val(serviceArrTmp[activeItem].camera_user);
  $('#pipCameraPass').val(serviceArrTmp[activeItem].camera_pass);

  // Active tab
  $('#pipSourceType .change_tab#' + serviceArrTmp[activeItem].source_type).tab('show');

  // Disable or enable FPS, Compression, Overlay size
  handleSourceTypeWithInputs(serviceArrTmp[activeItem].source_type);

  // Register event on tab click
  $('#pipSourceType .change_tab').off('click').on('click', function () {
    const tabId = $(this).attr('id');
    // Inputs display
    handleSourceTypeWithInputs(tabId);
    // Stop sharing if is not web camera (ws)
    if (tabId !== "ws") {
      sharingStop();
    }

    showSaveWarning();
  });
}

function sharingChangeOption() {
  $('#webcamSharingChange[data-service-id="' + serviceArrTmp[activeItem].id + '"]').show();
}

function handleWebcamSharingPreview(serviceId) {
  $('#webcamSharingShowPreview[data-service-id="' + serviceId + '"]').show();
}

function sharingOn() {
  $('#webcamSharingStart[data-service-id="' + serviceArrTmp[activeItem].id + '"]').hide();
  $('#webcamSharingStop[data-service-id="' + serviceArrTmp[activeItem].id + '"]').show();
  $('#stateCapture[data-service-id="' + serviceArrTmp[activeItem].id + '"]').show();
}

function sharingOff() {
  $('#webcamSharingChange[data-service-id="' + serviceArrTmp[activeItem].id + '"]').hide();
  $('#webcamSharingStart[data-service-id="' + serviceArrTmp[activeItem].id + '"]').show();
  $('#webcamSharingStop[data-service-id="' + serviceArrTmp[activeItem].id + '"]').hide();
  $('#stateCapture[data-service-id="' + serviceArrTmp[activeItem].id + '"]').hide();
}

function handleSourceTypeWithInputs(sourceType) {
  const sourceTypeInputToggles = {
    "axis_camera": { fps: false, compression: false, max_size: true },
    "mjpeg_url": { fps: true, compression: true, max_size: true },
    "ws": { fps: false, compression: true, max_size: false },
  }
  serviceArrTmp[activeItem].source_type = sourceType;

  $('#pipFPS').attr('disabled', sourceTypeInputToggles[sourceType].fps);
  $('#pipCompression').attr('disabled', sourceTypeInputToggles[sourceType].compression);
  $('#pipMaxSize').attr('disabled', sourceTypeInputToggles[sourceType].max_size);
}


function handleChangeMaxSize() {
  const maxSizeElement = $('#pipMaxSize');
  const minValue = maxSizeElement.attr('min');
  const maxSizeValue = Number(maxSizeElement.val());
  if (maxSizeValue >= minValue) {
    sharingInterface.setImageMaxSize(serviceArrTmp[activeItem].id, maxSizeValue);
  }
}

function handleChangeFps() {
  const fpsElement = $('#pipFPS');
  const minVal = fpsElement.attr('min');
  const maxVal = fpsElement.attr('max');
  const fpsValue = Number(fpsElement.val());
  if (fpsValue >= minVal && fpsValue <= maxVal) {
    sharingInterface.setFps(serviceArrTmp[activeItem].id, fpsValue);
  }
}

async function sharingChange() {
  await sharingStop();
  sharingStart();
}

async function sharingShowPreview() {
  await sharingInterface.showPreview(serviceArrTmp[activeItem].id);
  $('#webcamSharingShowPreview[data-service-id="' + serviceArrTmp[activeItem].id + '"]').hide();
}

async function sharingStart() {
  if (window.location.protocol != 'https:') {
    showMessage('warning', 'Webcam sharing is available only on HTTPS');
    return;
  }
  const serviceIsRunning = isServiceEnabled(serviceArrTmp[activeItem].id);
  if (!serviceIsRunning) {
    showMessage('warning', 'Webcam sharing service is currently disabled. Turn it on to start sharing.');
    return;
  }
  if (serviceArrTmp[activeItem].source_type !== serviceArr[activeItem].source_type) {
    showMessage('warning', 'Save your settings to start sharing your webcam.');
    return;
  }

  const url = new URL(window.location.origin);
  const wsConfig = {
    url: 'wss://' + url.hostname + '/local/camoverlay/service/ws',
    name: 'pip'
  }
  const resolution = cameraResolutions[parseInt($('#pipResolution').val())];
  const imageMaxSize = Number($('#pipMaxSize').val());
  const fps = Number($('#pipFPS').val());
  const typeSharing = 'webcam';
  const serviceId = serviceArrTmp[activeItem].id;

  try {
    await sharingInterface.start(
      typeSharing,
      wsConfig,
      serviceId,
      resolution,
      imageMaxSize,
      false,
      fps,
      sharingOff,
      sharingChangeOption,
      () => handleWebcamSharingPreview(serviceId),
    );
    sharingOn();

  } catch (err) {
    console.error(err);
    showMessage('danger', 'Web camera: ' + err.message);
  }
}

async function sharingStop() {
  sharingInterface.stop(serviceArrTmp[activeItem].id);
}

async function formChangedCallback() {
  serviceArrTmp[activeItem].customName = $('#pipCustomName').val();

  let currentCameraResolutionObject = cameraResolutions[parseInt($('#pipResolution').val())];
  serviceArrTmp[activeItem].width = currentCameraResolutionObject.width;
  serviceArrTmp[activeItem].height = currentCameraResolutionObject.height;
  await saveLayerZindex($('#layerZindex').val());
  serviceArrTmp[activeItem].coordSystem = $('#pipPosition').val();
  serviceArrTmp[activeItem].automationType = $('#pipAutomationTypeContainer input[type="radio"]:checked').val();
  serviceArrTmp[activeItem].pos_x = parseInt($('#pipPosX').val());
  serviceArrTmp[activeItem].pos_y = parseInt($('#pipPosY').val());
  serviceArrTmp[activeItem].fps = parseInt($('#pipFPS').val());
  if (serviceArrTmp[activeItem].fps < 1) {
    serviceArrTmp[activeItem].fps = 1;
    $('#pipFPS').val(serviceArrTmp[activeItem].fps);
  }

  serviceArrTmp[activeItem].compression = parseInt($('#pipCompression').val());
  serviceArrTmp[activeItem].screenSize = parseInt($('#pipMaxSize').val());

  serviceArrTmp[activeItem].mjpeg_url = $('#pipMjpegUrl').val().trim();

  serviceArrTmp[activeItem].camera_ip = $('#pipCameraIP').val();
  serviceArrTmp[activeItem].camera_port = parseInt($('#pipCameraPort').val());
  serviceArrTmp[activeItem].camera_user = $('#pipCameraUser').val();
  serviceArrTmp[activeItem].camera_pass = $('#pipCameraPass').val();

  if (!$('#pipCameraDisplayedOverlays').prop('disabled')) {
    serviceArrTmp[activeItem].camera_overlay_params = $('#pipCameraDisplayedOverlays').val();
  }

  if (propertiesResponse && $('#pipCameraViewArea').val() != null) {

    if (serviceArrTmp[activeItem].camera_view_area != $('#pipCameraViewArea').val()) {
      serviceArrTmp[activeItem].camera_view_area = $('#pipCameraViewArea').val();
      createCameraResolutionList(serviceArrTmp[activeItem].camera_view_area - 1, propertiesResponse);
    } else {
      serviceArrTmp[activeItem].camera_view_area = $('#pipCameraViewArea').val();
    }

    var cmResItem = $('#pipCameraResolution')[0];
    if (cmResItem.length > 0) {
      var res = cmResItem.options[cmResItem.selectedIndex].text;
      res = res.split('x');
      serviceArrTmp[activeItem].camera_width = parseInt(res[0].trim());
      serviceArrTmp[activeItem].camera_height = parseInt(res[1].trim());
    }
  }

  serviceArrTmp[activeItem].cameraList = getSelectedVideoChannelValues($('#pipVideoChannel'));

  showSaveWarning();
}

function waitForTimeout(timeout) {
  return new Promise(function (resolve) {
    setTimeout(resolve, timeout);
  });
}

async function getProxyCameraData(time) {
  const requestId = Math.random() * 1000;
  lastRequestId = requestId;

  $('#cameraInfoForm').showSpinner('Loading data');

  await waitForTimeout(time);

  // in case of calling this function multiple times
  // before previous are finished. Only execute the last request
  // For example in case of ip input change and check button click
  if (lastRequestId !== requestId) return;

  // check for camera firmware and conditionally display select
  try {
    await handleDisplayedOverlaySelect();
  } catch (error) {
    $('#cameraInfoForm').hideSpinner();
    throw new Error(error.message);
  }


  let configResponse;
  try {
    configResponse = await getProxyCameraParam('Image.NbrOfConfigs');
  } catch (error) {
    $('#cameraInfoForm').hideSpinner();
    throw new Error(error.message);
  }

  if (!configResponse.hasOwnProperty('Image.NbrOfConfigs')) {
    $('#cameraInfoForm').hideSpinner();
    throw new Error('Connection error 1 - can\'t get params of proxy camera');
  }

  let configCount = parseInt(configResponse['Image.NbrOfConfigs']);
  let params = 'Properties.Image.Resolution,Image.NbrOfConfigs';
  for (let i = 0; i < configCount; i++) {
    params += `,Properties.Image.I${i}.Resolution,Image.I${i}.Name,Image.I${i}.Enabled`;
  }

  let propsResponse;
  try {
    propsResponse = await getProxyCameraParam(params);
  } catch (error) {
    $('#cameraInfoForm').hideSpinner();
    throw new Error(error.message);
  }

  if (!propsResponse.hasOwnProperty('Properties.Image.Resolution')) {
    $('#cameraInfoForm').hideSpinner();
    throw new Error('Connection error 2 - can\'t get params of proxy camera');
  }

  propertiesResponse = propsResponse;
  configCount = parseInt(propsResponse['Image.NbrOfConfigs']);
  const selectedViewArea = createCameraViewAreaList(configCount, propertiesResponse); // Return selected view area number
  createCameraResolutionList(selectedViewArea, propertiesResponse);

  $('#cameraInfoForm').hideSpinner();
}

isRemoteCameraFirmwareCompatible = async (supportedFWVersion) => {
  const resVersion = await camOverlayApi.getRemoteDeviceInfo(
    {
      'method': 'getSupportedVersions'
    },
    {
      'x-target-camera-ip': serviceArrTmp[activeItem].camera_ip,
      'x-target-camera-port': serviceArrTmp[activeItem].camera_port,
      'x-target-camera-user': serviceArrTmp[activeItem].camera_user,
      'x-target-camera-pass': serviceArrTmp[activeItem].camera_pass
    }
  );

  const res = await camOverlayApi.getRemoteDeviceInfo(
    {
      'apiVersion': resVersion.apiVersions[0],
      'method': 'getProperties',
      'params': {
        'propertyList': [
          'Version'
        ]
      }
    },
    {
      'x-target-camera-ip': serviceArrTmp[activeItem].camera_ip,
      'x-target-camera-port': serviceArrTmp[activeItem].camera_port,
      'x-target-camera-user': serviceArrTmp[activeItem].camera_user,
      'x-target-camera-pass': serviceArrTmp[activeItem].camera_pass
    }
  );

  return res.propertyList.Version.localeCompare(supportedFWVersion, undefined, { numeric: true }) >= 1;
}

showDisplayedOverlaysSelect = () => {
  $('.pip-displayed-overlays').show();
  $('#pipCameraDisplayedOverlays').prop('disabled', false);
}

hideDisplayedOverlaysSelect = () => {
  $('.pip-displayed-overlays').hide();
  $('#pipCameraDisplayedOverlays').prop('disabled', true);
}

initDisplayedOverlaysSelect = () => {
  $('#pipCameraDisplayedOverlays').val(serviceArrTmp[activeItem].camera_overlay_params || "overlays=all");
  hideDisplayedOverlaysSelect();
}

handleDisplayedOverlaySelect = async () => {
  const compatibleVersion = await isRemoteCameraFirmwareCompatible(SUPPORTED_VERSION_FOR_REMOTE_PIP_OVERLAYS);
  if (compatibleVersion) {
    // show the input
    showDisplayedOverlaysSelect();
  } else {
    hideDisplayedOverlaysSelect();
  }
}

async function getProxyCameraParam(paramList) {
  return await camOverlayApi.getRemoteCameraParams(
    {
      'action': 'list',
      'group': paramList
    },
    {
      'x-target-camera-ip': serviceArrTmp[activeItem].camera_ip,
      'x-target-camera-port': serviceArrTmp[activeItem].camera_port,
      'x-target-camera-user': serviceArrTmp[activeItem].camera_user,
      'x-target-camera-pass': serviceArrTmp[activeItem].camera_pass
    }
  );
}

function parseResolutions(resString) {
  var resolutions = [];
  var resArr = resString.split(',');
  for (var i = 0; i < resArr.length; i++) {
    var resItems = resArr[i].split('x');
    if (resItems.length > 1) {
      resolutions.push({ "width": + parseInt(resItems[0]), "height": + parseInt(resItems[1]) });
    }
  }
  resolutions = resolutions.sort(function (a, b) {
    return b.width - a.width;
  });
  return resolutions;
}

function createCameraViewAreaList(viewAreaCount, cameraProperties) {
  var selected = -1;
  $('#pipCameraViewArea').empty();
  var resItem = $('#pipCameraViewArea')[0];
  for (var i = 0; i < viewAreaCount; i++) {
    if (cameraProperties[`Image.I${i}.Enabled`] == 'yes') {
      if (i + 1 == serviceArrTmp[activeItem].camera_view_area) {
        selected = i;
      }
      resItem.options[resItem.options.length] = new Option(cameraProperties[`Image.I${i}.Name`], i + 1);
    }
  }
  if (selected == -1) {
    selected = 0;
    if (resItem.options.length === 0) { // missing view areas in PTZ camera, add 'Default' value
      resItem.options[0] = new Option('Default', 1);
    }
    formChangedCallback();
  }
  resItem.selectedIndex = selected;
  return selected;
}

function createCameraResolutionList(viewArea, cameraProperties) {
  var resolutions = cameraProperties['Properties.Image.Resolution'];
  if (cameraProperties.hasOwnProperty(`Properties.Image.I${viewArea}.Resolution`)) {
    resolutions = cameraProperties[`Properties.Image.I${viewArea}.Resolution`];
  }
  resolutions = parseResolutions(resolutions);

  var selected = -1;
  $('#pipCameraResolution').empty();
  var resItem = $('#pipCameraResolution')[0];
  for (var i = 0; i < resolutions.length; i++) {
    if (resolutions[i].width == serviceArrTmp[activeItem].camera_width && resolutions[i].height == serviceArrTmp[activeItem].camera_height) {
      selected = i;
    }
    resItem.options[resItem.options.length] = new Option(resolutions[i].width + ' x ' + resolutions[i].height, i);
  }
  if (selected == -1) {
    selected = 0;
    formChangedCallback();
  }
  resItem.selectedIndex = selected;
  return selected;
}

function showConnectionState(state, text) {
  if (text !== undefined && text !== '') {
    if (state) {
      showMessage('success', text, false, 'pipConnectionStateSuccess');
      $("#cameraInfoForm").show();
    } else {
      showMessage('danger', text, false, 'pipConnectionStateError');
      $("#cameraInfoForm").hide();
    }
  }
}