var activePtzPosition = '';
var activeOverlay = 0;
var added = false;
var arrowIcon = '<span class="fas fa-arrow-alt-circle-right"></span>';

$(document).ready(async function () {

  initTooltips();
  initServiceId();

  $('#ptzScale').on('keyup change', function () {
    validateRange($(this), {
      minVal: 30,
      maxVal: 200,
      invalidMsg: 'Must be from interval 30-200%.',
      disableBtnId: 'saveOverlayBtn'
    });
  });

  $('#ptzDuration').on('keyup change', function () {
    validateRange($(this), {
      minVal: 0,
      maxVal: 99999,
      invalidMsg: 'Must be from interval 0-99999s.',
      disableBtnId: 'saveOverlayBtn'
    });
  });

  // Fill form
  initLayerZindex(serviceArrTmp[activeItem].zIndex || 0);

  $('#ptzCustomName')
    .val(serviceArrTmp[activeItem].customName || '')
    .keyup(onCustomNameKeyUp)
    .attachValidation({
      pattern: '^[A-Za-z0-9 -]{0,}$',
      invalidMsg: 'Only chars A-Z, a-z, 0-9, white space and dash are allowed.',
      disableBtnId: 'saveBtn'
    });

  const resolutionId = '#ptzResolution';
  const videoChannelId = '#ptzVideoChannel';
  const videoPreviewSelectId = '#ptzPreviewVideoChannel';
  createResolutionList(resolutionId);
  onResolutionChange(resolutionId, videoChannelId);
  await createTriggeringElement('#ptzAutomationTypeContainer', serviceArrTmp[activeItem].automationType, serviceArrTmp[activeItem].schedule, false);
  onVideoChannelChange(resolutionId, videoChannelId, videoPreviewSelectId);
  createVideoChannelElement(videoChannelId, serviceArrTmp[activeItem].cameraList, false).then(async function () {
    onPreviewVideoChannelChange(videoPreviewSelectId, resolutionId);
    // Callbacks for changes
    $('#mainForm .form-control').change(formChangedCallback);
    $('#saveOverlayBtn').click(saveEditedOverlayCallback);
    $('#editOverlayModalDismiss').click(cancelEditedOverlayCallback);
    $('#cancelOverlayBtn').click(cancelEditedOverlayCallback);
    $('#addOverlayBtn').click(addOverlayClickedCallback);

    renderPTZTable();

    $('#ptzImgPath').on('change', function () {
      ptzImagePathChanged();
    });

    formChangedCallback();

    let currentCameraResolutionObject = cameraResolutions[parseInt($('#ptzResolution').val())];
    showImageFromCamera('imagePreviewCamera', currentCameraResolutionObject, $(videoPreviewSelectId).val());
  });

  // specific listener for ptz service
  $(videoChannelId).change(renderPTZTable);

  // Import & export configuration
  configExport();
  configImport();

  createScheduleBtn();

  $('#ptzImageManagerBtn').click(showImageManagerCallback);
});

async function formChangedCallback() {

  let currentCameraResolutionObject = cameraResolutions[parseInt($('#ptzResolution').val())];
  serviceArrTmp[activeItem].width = currentCameraResolutionObject.width;
  serviceArrTmp[activeItem].height = currentCameraResolutionObject.height;
  await saveLayerZindex($('#layerZindex').val());
  serviceArrTmp[activeItem].automationType = $('#ptzAutomationTypeContainer input[type="radio"]:checked').val();

  serviceArrTmp[activeItem].cameraList = getSelectedVideoChannelValues($('#ptzVideoChannel'));

  showSaveWarning();
}

async function renderPTZTable() {
  const vcData = await getVideoChannelDataList();
  for (let i = 0; i < vcData.channelCount; i++) {
    if ((vcData.params[`Image.I${i}.Enabled`] === 'yes' || i === 0) && parseInt($('#ptzVideoChannel').val()) === i) {
      await createPtzList(i);
    }
  }
}

// creates ptz list for a given channel
async function createPtzList(cameraChannel) {

  const emptyTableTbody = '<tbody><td>No PTZ positions configured.</td></tbody>';
  const handleError = function () {
    $(`#ptzList`).html(emptyTableTbody);
    $('#ptzSettings').html('');
    $('#overlayList').html('');
  };

  if (isNaN(cameraChannel) || cameraChannel === -1) {
    handleError();
    return;
  }

  let ptzList;
  try {
    ptzList = await ptzManager.getList(cameraChannel);
  } catch (e) {
    handleError();
    return;
  }

  if (ptzList.length === 0) {
    $(`#ptzList`).html(emptyTableTbody);
  }

  let tableTbody = '<tbody>';
  for (let i = 0; i < ptzList.length; i++) {
    let ptzName = ptzList[i];
    let isSuccess = i === 0 ? ' success' : '';
    let isActive = i === 0 ? arrowIcon : '';
    tableTbody += '  <tr id="ptzRow' + i + '" data="' + ptzName + '" class="' + isSuccess + '">';
    tableTbody += '    <td class="text-left">' + ptzName + '</td>';
    tableTbody += '    <td class="text-right">' + isActive + '</td>';
    tableTbody += '  </tr>';

    iniPTZPositionsConfig(ptzName);
  }
  tableTbody += '</tbody>';
  $('#ptzList').html(tableTbody);

  $('#ptzList tr').click(ptzPositionClickedCallback);
  activePtzPosition = ptzList[0];
  setPtzPositionRowActive($('#ptzList tr:first-child'));
  showPtzOverlaySettings();
}

function setPtzPositionRowActive(element) {
  const index = Number(element.attr('id').substring(6));
  const data = element.attr('data').trim();
  const rows = $('tr');

  for (let i = 0; i < rows.length; i++) {
    const tdIcon = $(rows[i]).find('td.text-right');
    if (i === index) {
      rows[i].className = 'success';
      tdIcon.html(arrowIcon);
    } else {
      rows[i].className = '';
      tdIcon.html('');
    }
  }

  activePtzPosition = data;
  showPtzOverlaySettings();
  showSaveWarning();
}

function ptzPositionClickedCallback(event) {
  var row = $(event.target).closest('tr');
  setPtzPositionRowActive(row);
}

function iniPTZPositionsConfig(PtzPosition) {
  if (serviceArrTmp[activeItem].ptz_positions == undefined)
    serviceArrTmp[activeItem].ptz_positions = {};
  if (serviceArrTmp[activeItem].ptz_positions[PtzPosition] == undefined)
    serviceArrTmp[activeItem].ptz_positions[PtzPosition] = {};
  if (serviceArrTmp[activeItem].ptz_positions[PtzPosition].overlayList == undefined)
    serviceArrTmp[activeItem].ptz_positions[PtzPosition].overlayList = [];
  if (serviceArrTmp[activeItem].ptz_positions[PtzPosition].loop == undefined)
    serviceArrTmp[activeItem].ptz_positions[PtzPosition].loop = false;
}

function showPtzOverlaySettings() {
  var loop = serviceArrTmp[activeItem].ptz_positions[activePtzPosition].loop;
  var ptzSettings = '<form class="form-inline">';
  ptzSettings += '<div class="form-group">';
  ptzSettings += '<div class="checkbox-inline"><label class="formItem">';
  ptzSettings += '<input id="loopCheckbox" class="form-check-input image-control formItem" type="checkbox" value="" ' + (loop ? 'checked' : '') + '>Loop';
  ptzSettings += '</label></div></div></form>';
  $('#ptzSettings').html(ptzSettings);

  $('.image-control').change(imageFormChangedCallback);
  showOverlayList();
}

function imageFormChangedCallback() {
  serviceArrTmp[activeItem].ptz_positions[activePtzPosition].loop = $('#loopCheckbox').is(":checked");
  showSaveWarning();
}

function showOverlayList() {
  var overlayListHtml = '';

  var overlayList = serviceArrTmp[activeItem].ptz_positions[activePtzPosition].overlayList;
  if (overlayList != undefined && overlayList.length > 0) {
    overlayListHtml += '<tbody>';
    for (var i = 0; i < overlayList.length; i++) {
      overlayListHtml += '<tr>';
      var imgSrc = '/local/camoverlay/api/image.cgi?path=' + encodeURIComponent(overlayList[i].imgPath) + '&_t=' + Date.now();
      overlayListHtml += '<td style="max-width: 60px;"><img id="imageManagerPreview" src="' + imgSrc + '" style="max-width: 60px;" /></td>';
      overlayListHtml += '<td style="max-width: 120px; overflow: hidden; text-overflow: ellipsis; white-space: nowrap;>';
      overlayListHtml += '<span style="font-style: italic;">' + overlayList[i].imgName + '</span><br />';
      overlayListHtml += '</td>';

      overlayListHtml += '<td class="hidden-xs" style="text-align: center; max-width: 20px;">';
      if (overlayList[i].duration) {
        overlayListHtml += overlayList[i].duration + 's';
      } else {
        overlayListHtml += '0s';
      }
      overlayListHtml += '</td>';

      overlayListHtml += '<td class="hidden-xs" style="text-align: center; max-width: 110px;">';
      overlayListHtml += coordSystemToNiceName(overlayList[i].coordSystem) + '<br />';
      overlayListHtml += 'x: ' + overlayList[i].pos_x + ', ';
      overlayListHtml += 'y: ' + overlayList[i].pos_y;
      if (!isNaN(overlayList[i].scale)) {
        overlayListHtml += ', <br>scale: ' + convertFloatToPercentage(overlayList[i].scale) + '%';
      }
      overlayListHtml += '</td>';

      overlayListHtml += '<td style="text-align: center; max-width: 100px;">';
      overlayListHtml += '<a href="#" id="moveOverlay' + i + '" class="overlayMoveUp icon formItem ml-0"><span class="fas fa-arrow-circle-up" data-toggle="tooltip" title="Move Up"></span></a>';
      overlayListHtml += '<a href="#" id="moveOverlay' + i + '" class="overlayMoveDown icon formItem"><span class="fas fa-arrow-circle-down" data-toggle="tooltip" title="Move Down"></span></a>';
      overlayListHtml += '<a href="#" id="editOverlay' + i + '" class="overlayEdit formItem icon"><span class="fas fa-edit" data-toggle="tooltip" title="Edit"></span></a>';
      overlayListHtml += '<a href="#" id="removeOverlay' + i + '" class="overlayRemove formItem icon"><span class="fas fa-trash" data-toggle="tooltip" title="Delete"></span></a>';
      overlayListHtml += '</td></tr>';
    }
    overlayListHtml += '</tbody>';
  } else {
    serviceArrTmp[activeItem].overlayList = [];
    overlayListHtml += '<p class="text-center">No overlay set.</p>';
  }

  $('#overlayList').html(overlayListHtml);
  $('.overlayMoveUp').click(moveOverlayClicked);
  $('.overlayMoveDown').click(moveOverlayClicked);
  $('.overlayEdit').click(editOverlayClicked);
  $('.overlayRemove').click(removeOverlayClicked);

  showSaveWarning();
}

function addOverlayClickedCallback() {
  if (activePtzPosition.length == 0)
    return;

  activeOverlay = serviceArrTmp[activeItem].ptz_positions[activePtzPosition].overlayList.length;
  added = true;
  serviceArrTmp[activeItem].ptz_positions[activePtzPosition].overlayList.push(getDefaultOverlay());
  setEditedOverlay();
  $('#editOverlayModal').modal('show');
}

function getDefaultOverlay() {
  return { "coordSystem": "top_left", "pos_x": 10, "pos_y": 10, "imgPath": "", "imgName": "", "duration": 5, "scale": 1.0 };
}

async function setEditedOverlay() {
  var overlayList = serviceArrTmp[activeItem].ptz_positions[activePtzPosition].overlayList;
  createPositionList('#ptzPosition', overlayList[activeOverlay].coordSystem);
  $('#ptzPosX').val(overlayList[activeOverlay].pos_x);
  $('#ptzPosY').val(overlayList[activeOverlay].pos_y);
  let ptzScale = convertFloatToPercentage(overlayList[activeOverlay].scale);
  if (isNaN(ptzScale)) {
    ptzScale = 100;
  }
  $('#ptzScale').val(ptzScale);
  $('#ptzDuration').val(overlayList[activeOverlay].duration);

  // Create image list field
  await createImageList('#ptzImgPath', true, overlayList[activeOverlay].imgPath);
  ptzImagePathChanged();
}

function ptzImagePathChanged() {
  var options = $('#ptzImgPath option');
  var selectedIndex = $('#ptzImgPath')[0].selectedIndex;
  if (options.length === 0) {
    $('#ptzPreviewEditOverlay').html('');
    return;
  }
  var fileSrc = options[selectedIndex].getAttribute('value');
  $('#imagePreviewEditOverlay').html('<img id="imageManagerPreview" src="/local/camoverlay/api/image.cgi?path=' + encodeURIComponent(fileSrc) + '" />');
}

async function saveEditedOverlayCallback() {
  added = false;
  let imageList = await imageManager.getFileList();
  var image = findImageByPath(imageList, $('#ptzImgPath').val());
  serviceArrTmp[activeItem].ptz_positions[activePtzPosition].overlayList[activeOverlay].coordSystem = $('#ptzPosition').val();
  serviceArrTmp[activeItem].ptz_positions[activePtzPosition].overlayList[activeOverlay].pos_x = parseInt($('#ptzPosX').val());
  serviceArrTmp[activeItem].ptz_positions[activePtzPosition].overlayList[activeOverlay].pos_y = parseInt($('#ptzPosY').val());
  serviceArrTmp[activeItem].ptz_positions[activePtzPosition].overlayList[activeOverlay].imgPath = image.path;
  serviceArrTmp[activeItem].ptz_positions[activePtzPosition].overlayList[activeOverlay].imgName = image.name;
  serviceArrTmp[activeItem].ptz_positions[activePtzPosition].overlayList[activeOverlay].duration = parseInt($('#ptzDuration').val());
  serviceArrTmp[activeItem].ptz_positions[activePtzPosition].overlayList[activeOverlay].scale = convertPercentageToFloat($('#ptzScale').val(), 100);

  showPtzOverlaySettings(activePtzPosition);
}

function cancelEditedOverlayCallback() {
  if (added) {
    added = false;
    serviceArrTmp[activeItem].ptz_positions[activePtzPosition].overlayList.pop();
  } else {
    serviceArrTmp[activeItem].ptz_positions[activePtzPosition].overlayList[activeOverlay] = savedOverlay;
  }
  showPtzOverlaySettings(activePtzPosition);
}

function moveOverlayClicked(event) {
  var targ = event.target || event.srcElement;
  var linkElement = $(targ).closest('a');
  var index = linkElement.attr('id').substring(11);
  var newIndex = parseInt(index) + 1;
  if (linkElement.hasClass('overlayMoveUp'))
    newIndex = parseInt(index) - 1;

  var overlayList = serviceArrTmp[activeItem].ptz_positions[activePtzPosition].overlayList;
  if (newIndex < 0 || newIndex >= overlayList.length)
    return false;
  var tmp = overlayList[index];
  overlayList[index] = overlayList[newIndex];
  overlayList[newIndex] = tmp;
  showOverlayList();
  return false;
}

function editOverlayClicked(event) {
  var targ = event.target || event.srcElement;
  var index = $(targ).closest('a').attr('id').substring(11);
  activeOverlay = index;
  savedOverlay = serviceArrTmp[activeItem].ptz_positions[activePtzPosition].overlayList[activeOverlay];
  setEditedOverlay();
  $('#editOverlayModal').modal('show');
  return false;
}

function removeOverlayClicked(event) {
  var targ = event.target || event.srcElement;
  var index = $(targ).closest('a').attr('id').substring(13);
  serviceArrTmp[activeItem].ptz_positions[activePtzPosition].overlayList.splice(index, 1);
  showPtzOverlaySettings(activePtzPosition);
  return false;
}