$(document).ready(async function () {
  initTooltips();
  initServiceId();

  // Fill form
  initLayerZindex(serviceArrTmp[activeItem].zIndex || 0);

  $('#screenSharingCustomName')
    .val(serviceArrTmp[activeItem].customName || '')
    .keyup(onCustomNameKeyUp)
    .attachValidation({
      pattern: '^[A-Za-z0-9 -]{0,}$',
      invalidMsg: 'Only chars A-Z, a-z, 0-9, white space and dash are allowed.',
      disableBtnId: 'saveBtn'
    });

  $('#screenSharingPosX').val(serviceArrTmp[activeItem].pos_x);
  $('#screenSharingPosY').val(serviceArrTmp[activeItem].pos_y);

  $('#screenSharingFPS').val(serviceArrTmp[activeItem].fps || 1);
  $('#screenSharingMaxSize').val(serviceArrTmp[activeItem].screenSize || 500);

  const resolutionId = '#screenSharingResolution';
  const videoChannelId = '#screenSharingVideoChannel';
  const videoPreviewSelectId = '#screenSharingPreviewVideoChannel';
  createResolutionList(resolutionId);
  onResolutionChange(resolutionId, videoChannelId);
  createPositionList('#screenSharingPosition', serviceArrTmp[activeItem].coordSystem);
  await createTriggeringElement('#screenSharingAutomationTypeContainer', serviceArrTmp[activeItem].automationType, serviceArrTmp[activeItem].schedule, false);
  onVideoChannelChange(resolutionId, videoChannelId, videoPreviewSelectId);
  await createVideoChannelElement(videoChannelId, serviceArrTmp[activeItem].cameraList);
  onPreviewVideoChannelChange(videoPreviewSelectId, resolutionId);

  let currentCameraResolutionObject = cameraResolutions[parseInt($('#screenSharingResolution').val())];
  showImageFromCamera('imagePreviewCamera', currentCameraResolutionObject, $(videoPreviewSelectId).val());

  // Callbacks for changes
  $('.form-control').change(formChangedCallback);
  $('#screenSharingChange').attr('data-service-id', serviceArrTmp[activeItem].id).click(sharingChangeScreen);
  $('#screenSharingShowPreview').attr('data-service-id', serviceArrTmp[activeItem].id).click(sharingShowPreview)
  $('#screenSharingStart').attr('data-service-id', serviceArrTmp[activeItem].id).click(sharingStart);
  $('#screenSharingStop').attr('data-service-id', serviceArrTmp[activeItem].id).click(sharingStop);
  $('#fullscreenContainer').attr('data-service-id', serviceArrTmp[activeItem].id);
  $('#stateCapture').attr('data-service-id', serviceArrTmp[activeItem].id);
  $('#fullscreenOn').attr('data-service-id', serviceArrTmp[activeItem].id).click(handleFullscreenOn);
  $('#fullscreenOff').attr('data-service-id', serviceArrTmp[activeItem].id).click(handleFullscreenOff);
  $('#screenSharingMaxSize')
    .off('change')
    .change(handleChangeMaxSize)
    .attachValidation({
      pattern: '^[1-9][0-9]{0,}$',
      invalidMsg: 'The value must not be empty or zero.',
      disableBtnId: 'screenSharingStart'
    });
  $('#screenSharingFPS')
    .change(handleChangeFps)
    .attachValidation({
      pattern: '^([1-9]|[12][0-9]|30)$',
      invalidMsg: 'The value must be 1 to 30.',
      disableBtnId: 'screenSharingStart'
    });

  // Handle screen sharing state
  const isRunning = sharingInterface.isRunning(serviceArrTmp[activeItem].id);
  if (isRunning) {
    handleRunningSharing();
  }

  const isVisiblePreview = await sharingInterface.isPreviewVisible(serviceArrTmp[activeItem].id);
  if (isRunning && !isVisiblePreview) {
    $('#screenSharingShowPreview[data-service-id="' + serviceArrTmp[activeItem].id + '"]').show();
  }

  formChangedCallback();
  createScheduleBtn();
});

function handleRunningSharing() {
  if (sharingInterface.getFullscreen(serviceArrTmp[activeItem].id)) {
    fullscreenTurnOn();
  } else {
    fullscreenTurnOff();
  }
  sharingOn();
}

async function handleChangeMaxSize() {
  const maxSizeElement = $('#screenSharingMaxSize');
  const minValue = maxSizeElement.attr('min');
  const maxSizeValue = Number(maxSizeElement.val());
  if (maxSizeValue >= minValue) {
    serviceArrTmp[activeItem].screenSize = maxSizeValue;
    serviceArr[activeItem].screenSize = maxSizeValue;
    sharingInterface.setImageMaxSize(serviceArrTmp[activeItem].id, maxSizeValue);
    await saveBtnCallback(true, serviceArrTmp);
  }
}

function handleChangeFps() {
  const fpsElement = $('#screenSharingFPS');
  const minVal = fpsElement.attr('min');
  const maxVal = fpsElement.attr('max');
  const fpsValue = Number(fpsElement.val());
  if (fpsValue >= minVal && fpsValue <= maxVal) {
    sharingInterface.setFps(serviceArrTmp[activeItem].id, fpsValue);
  }
}

function fullscreenTurnOn() {
  $('#fullscreen').prop('checked', true);
  $('#fullscreenOn[data-service-id="' + serviceArrTmp[activeItem].id + '"]').hide();
  $('#fullscreenOff[data-service-id="' + serviceArrTmp[activeItem].id + '"]').show();
}

function fullscreenTurnOff() {
  $('#fullscreen').prop('checked', false);
  $('#fullscreenOff[data-service-id="' + serviceArrTmp[activeItem].id + '"]').hide();
  $('#fullscreenOn[data-service-id="' + serviceArrTmp[activeItem].id + '"]').show();
}

function handleFullscreenOn() {
  sharingInterface.setFullscreen(serviceArrTmp[activeItem].id, true);
  fullscreenTurnOn();
}

function handleFullscreenOff() {
  sharingInterface.setFullscreen(serviceArrTmp[activeItem].id, false);
  fullscreenTurnOff();
}

function handleScreenSharingPreview(serviceId) {
  $('#screenSharingShowPreview[data-service-id="' + serviceId + '"]').show();
}

function sharingOn() {
  $('#screenSharingStart[data-service-id="' + serviceArrTmp[activeItem].id + '"]').hide();
  $('#screenSharingChange[data-service-id="' + serviceArrTmp[activeItem].id + '"]').show();
  $('#screenSharingStop[data-service-id="' + serviceArrTmp[activeItem].id + '"]').show();
  $('#fullscreenContainer[data-service-id="' + serviceArrTmp[activeItem].id + '"]').show();
  $('#stateCapture[data-service-id="' + serviceArrTmp[activeItem].id + '"]').show();
}

function sharingOff() {
  $('#screenSharingStart[data-service-id="' + serviceArrTmp[activeItem].id + '"]').show();
  $('#screenSharingChange[data-service-id="' + serviceArrTmp[activeItem].id + '"]').hide();
  $('#screenSharingStop[data-service-id="' + serviceArrTmp[activeItem].id + '"]').hide();
  $('#screenSharingShowPreview[data-service-id="' + serviceArrTmp[activeItem].id + '"]').hide();
  $('#fullscreenContainer[data-service-id="' + serviceArrTmp[activeItem].id + '"]').hide();
  $('#stateCapture[data-service-id="' + serviceArrTmp[activeItem].id + '"]').hide();
}

async function sharingStart() {
  if (window.location.protocol != 'https:') {
    showMessage('warning', 'Screen sharing is available only on HTTPS');
    return;
  }
  const serviceIsRunning = isServiceEnabled(serviceArrTmp[activeItem].id);
  if (!serviceIsRunning) {
    showMessage('warning', 'Screen sharing service is currently disabled. Turn it on to start sharing.');
    return;
  }

  const url = new URL(window.location.origin);
  const wsConfig = {
    url: 'wss://' + url.hostname + '/local/camoverlay/service/ws',
    name: 'screen-sharing'
  }
  const resolution = cameraResolutions[parseInt($('#screenSharingResolution').val())];
  const imageMaxSize = Number($('#screenSharingMaxSize').val());
  const fullscreenOn = Boolean($('#fullscreen').is(':checked'));
  const fps = Number($('#screenSharingFPS').val());
  const typeSharing = 'screen';
  const serviceId = serviceArrTmp[activeItem].id;

  try {
    await sharingInterface.start(
      typeSharing,
      wsConfig,
      serviceId,
      resolution,
      imageMaxSize,
      fullscreenOn,
      fps,
      sharingOff,
      () => { },
      () => handleScreenSharingPreview(serviceId),
    );
    sharingOn();

  } catch (err) {
    console.error(err);
    showMessage('danger', 'Screen Sharing: ' + err.message);
  }
}

async function sharingChangeScreen() {
  await sharingStop();
  sharingStart();
}

async function sharingShowPreview() {
  await sharingInterface.showPreview(serviceArrTmp[activeItem].id);
  $('#screenSharingShowPreview[data-service-id="' + serviceArrTmp[activeItem].id + '"]').hide();
}

async function sharingStop() {
  sharingInterface.stop(serviceArrTmp[activeItem].id);
}

async function formChangedCallback() {
  serviceArrTmp[activeItem].customName = $('#screenSharingCustomName').val();

  let currentCameraResolutionObject = cameraResolutions[parseInt($('#screenSharingResolution').val())];
  serviceArrTmp[activeItem].width = currentCameraResolutionObject.width;
  serviceArrTmp[activeItem].height = currentCameraResolutionObject.height;
  await saveLayerZindex($('#layerZindex').val());
  serviceArrTmp[activeItem].coordSystem = $('#screenSharingPosition').val();
  serviceArrTmp[activeItem].automationType = $('#screenSharingAutomationTypeContainer input[type="radio"]:checked').val();
  serviceArrTmp[activeItem].pos_x = parseInt($('#screenSharingPosX').val());
  serviceArrTmp[activeItem].pos_y = parseInt($('#screenSharingPosY').val());
  serviceArrTmp[activeItem].fps = parseInt($('#screenSharingFPS').val());
  serviceArrTmp[activeItem].screenSize = parseInt($('#screenSharingMaxSize').val());

  if (serviceArrTmp[activeItem].fps < 1) {
    serviceArrTmp[activeItem].fps = 1;
    $('#screenSharingFPS').val(serviceArrTmp[activeItem].fps);
  }

  serviceArrTmp[activeItem].cameraList = getSelectedVideoChannelValues($('#screenSharingVideoChannel'));

  showSaveWarning();
}